searchData={"items":[{"type":"task","title":"mix prom_ex.dashboard.export","doc":"This will render a PromEx dashboard either to STDOUT or to a file depending on\nthe CLI arguments that are provided.\n\nThe following CLI flags are supported:\n```md\n-d, --dashboard  The name of the dashboard that you would like to export from PromEx.\n                 For example, if you would like to export the Ecto dashboard, provide\n                 the value `ecto.json`.\n\n-m, --module     The PromEx module which will be used to render the dashboards.\n                 This is needed to fetch any relevant assigns from the\n                 `c:PromEx.dashboard_assigns/0` callback\n\n-s, --stdout    A boolean flag denoting that the rendered dashboard should be output\n                 to STDOUT.\n\n-f, --file_path  If you would like the write the generated JSON dashboard definition\n                 to a file, you can provide a relative file path in the project's\n                 `priv` directory.\n\n-a, --assign     Any additional assigns you would like to pass to the dashboard for\n                 rendering. You are able to pass multiple assigns by passing multiple\n                 --assign arguments. For example: `--assign some=thing --assign another=thing`.\n```","ref":"Mix.Tasks.PromEx.Dashboard.Export.html"},{"type":"task","title":"mix prom_ex.dashboard.lint","doc":"This will lint Grafana dashboards to ensure that PromEx dashboards\nmeeting certain requirements.","ref":"Mix.Tasks.PromEx.Dashboard.Lint.html"},{"type":"task","title":"mix prom_ex.dashboard.publish","doc":"This mix task will publish dashboards to Grafana for a PromEx module. It is\nrecommended that you use the functionality that is part of the PromEx supervision\ntree in order to upload dashboards as opposed to this, given that mix may not\nalways be available (like in a mix release). This is more so a convenience for\ntesting and validating dashboards without starting the whole application.\n\nThe following CLI flags are supported:\n```md\n-m, --module   The PromEx module which will be used to render the dashboards.\n               This is needed to fetch any relevant assigns from the\n               `c:PromEx.dashboard_assigns/0` callback and to get the Grafana\n               configuration from app config.\n\n-t, --timeout  The timeout value defines how long the mix task will wait while\n               uploading dashboards.\n```","ref":"Mix.Tasks.PromEx.Dashboard.Publish.html"},{"type":"task","title":"mix prom_ex.gen.config","doc":"This Mix Task generates a PromEx config module in your project. This config\nfile acts as a starting point with instructions on how to set up PromEx\nin your application, some default PromEx metrics plugins, and their\naccompanying dashboards.\n\nThe following CLI flags are supported:\n```md\n-d, --datasource      The datasource that the dashboards will be reading from to populate\n                      their time series data. This `datasource` value should align with\n                      what is configured in Grafana from the Prometheus instance's\n                      `datasource_id`.\n\n-o, --otp_app         The OTP application that PromEx is being installed in. This\n                      should be provided as the snake case atom (minus the leading\n                      colon). For example, if the `:app` value in your `mix.exs` file\n                      is `:my_cool_app`, this argument should be provided as `my_cool_app`.\n                      By default PromEx will read your `mix.exs` file to determine the OTP\n                      application value so this is an OPTIONAL argument.\n\n-m, --prom_ex_module  Optional name of the PromEx module that will be created. This should\n                      be provided as an unscoped module name, and it will be saved to the\n                      corresponding file, e.g. `-m OpsPromEx` will save to `ops_prom_ex.ex`.\n\n```","ref":"Mix.Tasks.PromEx.Gen.Config.html"},{"type":"behaviour","title":"PromEx","doc":"PromEx is a plugin based library which can be used to capture\ntelemetry events and report them out for consumption by Prometheus.\nThe main purpose of this particular library is to provide the\nbehaviour that all PromEx plugins leverage so that a consistent\ninterface can be achieved and so that leveraging multiple plugins is\neffortless from the user's point of view.\n\nTo use PromEx you need to define a module that uses the PromEx library. This module\nwill also need to have some application config set for it similarly to how Ecto does.\nFor example, for a PromEx module defined like so:\n\n```elixir\ndefmodule MyApp.PromEx do\n  use PromEx,  otp_app: :web_app\n\n  ...\nend\n```\n\nYou would have an application configuration set like so:\n\n```elixir\nconfig :my_app, MyApp.PromEx,\n  manual_metrics_start_delay: :no_delay,\n  drop_metrics_groups: [],\n  grafana: [\n    host: System.get_env(\"GRAFANA_HOST\", \"http://grafana:3000\"),\n    auth_token: System.get_env(\"GRAFANA_TOKEN\", \"\"),\n    upload_dashboards_on_start: true,\n    folder_name: \"My App Dashboards\",\n    annotate_app_lifecycle: true\n  ]\n```\n\nThe options that you can pass to PromEx macro are outlined in the following section. In order\nto tell PromEx what plugins you would like to use and what dashboards you would like PromEx\nto upload for you, implement the `plugins/0` and `dashboards/0` callbacks respectively. The\n`dashboard_assigns/0` callback will be used when your EEx template Grafana dashboards are\nrendered so that the dashboards that are created for your application coincide with the PromEx\nconfiguration for the application. If your dashboards are not EEx templates, then the dashboard\nassigns are not passed through. Each plugin also has an accompanying Grafana dashboard that you\ncan leverage to plot all of the plugin captured data.\n\nIn order to expose captured metrics, you can leverage the PromEx provided Plug `PromEx.Plug`.\nSee the `PromEx.Plug` documentation modules for specifics on how to use it.","ref":"PromEx.html"},{"type":"behaviour","title":"Options - PromEx","doc":"* `:otp_app` - This is a REQUIRED field and is used by PromEx to fetch the application\n  configuration values for the various PromEx capture modules. Make sure that this value\n  matches the `:app` value in `project/0` from your `mix.exs` file. If you use the PromEx\n  `mix prom_ex.create` mix task this will be done automatically for you.","ref":"PromEx.html#module-options"},{"type":"behaviour","title":"Top level PromEx Configuration - PromEx","doc":"PromEx is generally configured per application per PromEx module as you saw in the previous section. The only\nsetting that is currently available globally for all instances of PromEx is that of the storage adapter. The\nstorage adapter configuration determines how PromEx captures and stores metrics. There are currently two\navailable adapters:\n\n1. [TelemetryMetricsPrometheus.Core](https://github.com/beam-telemetry/telemetry_metrics_prometheus_core) - This is the\ndefault adapter and is included with PromEx.\n2. [Peep](https://github.com/rkallos/peep) - In order to use Peep as your storage adapter,  you will need to add the following\nto your `config.exs` file:\n\n    ```elixir\n    config :prom_ex, :storage_adapter, PromEx.Storage.Peep\n    ```","ref":"PromEx.html#module-top-level-promex-configuration"},{"type":"behaviour","title":"PromEx Plugins - PromEx","doc":"All metrics collection will be delegated to plugins which can be found here:\n\nFoundational metrics:\n  - [X] `PromEx.Plugins.Application` Application related informational metrics\n  - [X] `PromEx.Plugins.Beam` BEAM virtual machine metrics\n  - [ ] Operating System (http://erlang.org/doc/man/os_mon_app.html)\n\nLibrary metrics:\n  - [X] `PromEx.Plugins.Ecto` - [Telemetry docs](https://hexdocs.pm/ecto/Ecto.Repo.html#module-telemetry-events)\n  - [X] `PromEx.Plugins.Oban` - [Telemetry docs](https://hexdocs.pm/oban/Oban.Telemetry.html)\n  - [X] `PromEx.Plugins.Phoenix` - [Telemetry docs](https://hexdocs.pm/phoenix/Phoenix.Logger.html)\n  - [X] `PromEx.Plugins.PhoenixLiveView` - [Telemetry docs](https://hexdocs.pm/phoenix_live_view/telemetry.html)\n  - [X] `PromEx.Plugins.Absinthe` - [Telemetry docs](https://hexdocs.pm/absinthe/1.5.3/telemetry.html)\n  - [X] `PromEx.Plugins.PlugCowboy` - [Telemetry docs](https://hexdocs.pm/plug_cowboy/2.4.0/Plug.Cowboy.html#module-instrumentation)\n  - [X] `PromEx.Plugins.PlugRouter` - [Telemetry docs](https://hexdocs.pm/plug/1.12.1/Plug.Router.html#module-telemetry)\n  - [X] `PromEx.Plugins.Broadway` - [Telemetry docs](https://hexdocs.pm/broadway/Broadway.html#module-telemetry)\n\nBacklog Elixir library metrics:\n  - [ ] Finch - [Telemetry docs](https://hexdocs.pm/finch/Finch.Telemetry.html#content)\n  - [ ] Swoosh - [Telemetry docs](https://hexdocs.pm/swoosh/1.5.0/Swoosh.html#module-telemetry)\n  - [ ] ChromicPDF - [Telemetry docs](https://hexdocs.pm/chromic_pdf/ChromicPDF.html#module-telemetry-support)\n  - [ ] Dataloader - [Telemetry docs](https://hexdocs.pm/dataloader/telemetry.html)\n  - [ ] GenRMQ - [Telemetry docs](https://hexdocs.pm/gen_rmq/3.0.0/GenRMQ.Publisher.Telemetry.html and https://hexdocs.pm/gen_rmq/3.0.0/GenRMQ.Consumer.Telemetry.html)\n  - [ ] Plug - [Telemetry docs](https://hexdocs.pm/plug/Plug.Telemetry.html)\n  - [ ] Redix - [Telemetry docs](https://hexdocs.pm/redix/Redix.Telemetry.html)\n  - [ ] Tesla - [Telemetry docs](https://hexdocs.pm/tesla/Tesla.Middleware.Telemetry.html)\n  - [ ] Memcachex - [Telemetry docs](https://hexdocs.pm/memcachex/0.5.0/Memcache.html#module-telemetry)\n  - [ ] Nebulex - [Telemetry docs](https://hexdocs.pm/nebulex/2.0.0-rc.0/telemetry.html)\n  - [ ] Horde - [Telemetry docs](https://github.com/derekkraan/horde/blob/master/lib/horde/supervisor_telemetry_poller.ex)\n  - [ ] Cachex - (Need to open up PR)\n  - [ ] Quantum - [Telemetry docs](https://hexdocs.pm/quantum/3.3.0/telemetry.html#content)\n  - [ ] ETS - [Erlang docs](https://www.erlang.org/doc/man/ets.html#info-1)\n\nDatabase cron based metrics:\n  - [ ] Postgres (https://github.com/pawurb/ecto_psql_extras for inspiration)\n  - [ ] Mnesia (https://github.com/deadtrickster/prometheus.erl/blob/master/src/collectors/mnesia/prometheus_mnesia_collector.erl for inspiration)\n  - [ ] MySQL (https://github.com/prometheus/mysqld_exporter for inspiration)\n  - [ ] Redis (https://github.com/oliver006/redis_exporter for inspiration)\n  - [ ] MongoDB (https://github.com/percona/mongodb_exporter for inspiration)","ref":"PromEx.html#module-promex-plugins"},{"type":"callback","title":"PromEx.dashboard_assigns/0","doc":"","ref":"PromEx.html#c:dashboard_assigns/0"},{"type":"callback","title":"PromEx.dashboards/0","doc":"","ref":"PromEx.html#c:dashboards/0"},{"type":"function","title":"PromEx.get_metrics/1","doc":"A simple pass-through to fetch all of the currently configured metrics. This is\nprimarily used by the exporter plug to fetch all of the metrics so that they\ncan be scraped.","ref":"PromEx.html#get_metrics/1"},{"type":"callback","title":"PromEx.init_opts/0","doc":"","ref":"PromEx.html#c:init_opts/0"},{"type":"callback","title":"PromEx.plugins/0","doc":"","ref":"PromEx.html#c:plugins/0"},{"type":"type","title":"PromEx.dashboard_definition/0","doc":"","ref":"PromEx.html#t:dashboard_definition/0"},{"type":"type","title":"PromEx.measurements_mfa/0","doc":"","ref":"PromEx.html#t:measurements_mfa/0"},{"type":"type","title":"PromEx.plugin_definition/0","doc":"","ref":"PromEx.html#t:plugin_definition/0"},{"type":"type","title":"PromEx.telemetry_metrics/0","doc":"","ref":"PromEx.html#t:telemetry_metrics/0"},{"type":"module","title":"PromEx.BucketGenerator","doc":"This module provides functions to generate histogram bucket ranges.\nThe lists of buckets that can be generated are either linear\nor exponential.","ref":"PromEx.BucketGenerator.html"},{"type":"function","title":"PromEx.BucketGenerator.exponential!/3","doc":"Create an exponential set of buckets based on the provided parameters.","ref":"PromEx.BucketGenerator.html#exponential!/3"},{"type":"function","title":"Examples - PromEx.BucketGenerator.exponential!/3","doc":"iex> PromEx.BucketGenerator.exponential!(1, 4, 10)\n    [1, 4, 16, 64, 256, 1024, 4096, 16384, 65536, 262144]\n\n    iex> PromEx.BucketGenerator.exponential!(1, 2, 10)\n    [1, 2, 4, 8, 16, 32, 64, 128, 256, 512]","ref":"PromEx.BucketGenerator.html#exponential!/3-examples"},{"type":"function","title":"PromEx.BucketGenerator.linear!/3","doc":"Create a linear set of buckets based on the provided parameters.","ref":"PromEx.BucketGenerator.html#linear!/3"},{"type":"function","title":"Examples - PromEx.BucketGenerator.linear!/3","doc":"iex> PromEx.BucketGenerator.linear!(10, 10, 10)\n    [10, 20, 30, 40, 50, 60, 70, 80, 90, 100]\n\n    iex> PromEx.BucketGenerator.linear!(0, 250, 11)\n    [0, 250, 500, 750, 1000, 1250, 1500, 1750, 2000, 2250, 2500]","ref":"PromEx.BucketGenerator.html#linear!/3-examples"},{"type":"module","title":"PromEx.Config","doc":"This module defines a struct that contains all of the fields necessary to configure\nan instance of PromEx.\n\nWhile this module does not directly access your Application config, PromEx will call the\n`PromEx.Config.build/1` function directly with the contents of `Application.get_env(:your_otp_app, YourPromEx.Module)`. As\nsuch, this is an appropriate place to talk about how you go about configuring PromEx via your Application config.\n\nBy default, you can run PromEx without any additional configuration and PromEx will fall back on some sane defaults. Specifically,\nif you were to not add any configuration to your config.exs, dev.exs, prod.exs, etc files it would be the same as setting the\nfollowing config:\n\n```elixir\nconfig :web_app, WebApp.PromEx,\n  disabled: false,\n  manual_metrics_start_delay: :no_delay,\n  drop_metrics_groups: [],\n  grafana: :disabled,\n  metrics_server: :disabled\n```\n\nIn this configuration, the Grafana dashboards are not uploaded on application start, and a standalone HTTP metrics server is not\nstarted. In addition, the `PromEx.ManualMetricsManager` is started without any time delay, and all metrics groups from all the plugins\nare registered and set up.\n\nIf you would like to set up PromEx to communicate with Grafana, your config would look something like:\n\n```elixir\nconfig :web_app, WebApp.PromEx,\n  grafana: [\n    host: \"http://localhost:3000\",\n    username: \" \",  # Authenticate via Basic Auth\n    password: \" \",\n    auth_token: \" \", # Or authenticate via API Token\n    upload_dashboards_on_start: true # This is an optional setting and will default to `true`\n  ]\n  ```\n\nIf you would like PromEx to start a standalone HTTP server to serve your aggregated metrics, you can leverage the `:metrics_server`\nconfig:\n\n```elixir\nconfig :web_app, WebApp.PromEx,\n  metrics_server: [\n    port: 4021,\n    path: \"/metrics\", # This is an optional setting and will default to `\"/metrics\"`\n    protocol: :http, # This is an optional setting and will default to `:http`\n    pool_size: 5, # This is an optional setting and will default to `5`\n    cowboy_opts: [], # This is an optional setting and will default to `[]`\n    auth_strategy: :none # This is an optional and will default to `:none`\n  ]\n```\n\nIf you would like the metrics server to be protected behind some sort of authentication, you can configure your `:metrics_server`\nlike so:\n\n```elixir\nconfig :web_app, WebApp.PromEx,\n  metrics_server: [\n    port: 4021,\n    auth_strategy: :bearer,\n    auth_token: \"VGhpcyBpcyBzdXBlciBzZWNyZXQuLi5kb24ndCBkZWNvZGUgbWU=\"\n  ]\n```","ref":"PromEx.Config.html"},{"type":"module","title":"Option Details - PromEx.Config","doc":"* `:disabled` - This option will disable the PromEx supervision tree entirely and will not\n  start any metrics collectors. This is primarily used for disabling PromEx during testing. Default\n  value: false\n\n* `:manual_metrics_start_delay` - Manual metrics are gathered once on start up and then only when\n  you call `PromEx.ManualMetricsManager.refresh_metrics/1`. Sometimes, you may have metrics\n  that require your entire supervision tree to be started in order to fetch accurate data.\n  This option will allow you to delays the initial metrics capture of the\n  `ManualMetricsManager` by a certain number of milliseconds or the `:no_delay` atom if you\n  want the metrics to be captured as soon as the `ManualMetricsManager` starts up. Default\n  value: `:no_delay`\n\n* `:drop_metrics_groups` - A list of all the metrics groups that you are not interested in\n  tracking. For example, if your application does not leverage Phoenix channels at all but\n  you still would like to use the `PromEx.Plugins.Phoenix` plugin, you can pass\n  `[:phoenix_channel_event_metrics]` as the value to `:drop_metrics_groups` and that set of\n  metrics will not be captured. Default value: `[]`\n\n* `ets_flush_interval` - This value denotes how often the metrics ETS table is compacted. In order\n  to keep things performant and as low-overhead as possible, Telemetry metrics are buffered up in\n  ETS until a request is made to retrieve metrics from the PromEx process. If no requests come in\n  to extract the metrics, the ETS table can grow infinitely. Luckily, PromEx bundles a GenServer\n  that periodically compacts ETS. This config value determines how often ETS should be compacted.\n  Default value: `7_500`\n\n* `:grafana` - This key contains the configuration information for connecting to Grafana. Its\n  configuration options are:\n\n  * `:host` - The host address of your Grafana instance. In order for PromEx to communicate with\n    Grafana this value should be in the format `protocol://host:port` like `http://localhost:3000`\n    for example.\n\n  * `:username` - The username that was created in Grafana so that PromEx can upload dashboards\n    via the API.\n\n  * `:password` - The password that was created in Grafana so that PromEx can upload dashboards\n    via the API.\n\n  * `:auth_token` - The auth token that was created in Grafana so that PromEx can upload dashboards\n    via the API.\n\n  * `:upload_dashboards_on_start` - Using the config values that you set in your application config\n    (`config.exs`, `dev.exs`, `prod.exs`, etc) PromEx will attempt to upload your Dashboards to\n    Grafana using Grafana's HTTP API.\n\n  * `:folder_name` - The name of the folder that PromEx will put all of the project dashboards in.\n    PromEx will automatically generate a unique ID for the folder based on the project's otp_app\n    value so that it can access the correct folder in Grafana. This also makes sure that different\n    Elixir projects running in the same cluster and publishing dashboards to Grafana do not collide\n    with one another. If no name is provided, then the dashboards will all be uploaded to the default\n    Grafana folder.\n\n  * `:annotate_app_lifecycle` - By enabling this setting, PromEx will leverage the Grafana API to annotate\n    when the application was started, and when it was shut down. By default this is disabled but if you\n    do enable it, no action is required from you in order to display these events on the dashboards. The\n    annotations will automatically contain the necessary tags to only display on the PromEx dashboards.\n    The annotation will include information including:\n      - Hostname\n      - OTP app name\n      - App version\n      - Git SHA of the last commit (if the GIT_SHA environment variable is present)\n      - Git author of the last commit (if the GIT_AUTHOR environment variable is present)\n\n  * `:finch_pools` - (optional) A map that will be passed to Finch.start_link/1 as the :pools key,\n    which can be used to configure protocol, pool size, HTTP host headers, proxy server, etc.\n\n* `:grafana_agent` - This key contains the configuration information for running GrafanaAgent via a\n  port in order to push metrics to a Prometheus instance via `remote_write` functionality:\n\n  >","ref":"PromEx.Config.html#module-option-details"},{"type":"module","title":"Environment dependencies {: .warning} - PromEx.Config","doc":">\n  > If your application is running inside of an Alpine Linux container (or any environment that\n  > is based on [musl](https://www.musl-libc.org/) as opposed to\n  > [glibc](https://www.gnu.org/software/libc/), be sure to add `libc6-compat` to to your list\n  > of packages. In addition, you'll also need bash running, as this port is wrapped by a\n  > [bash script](https://hexdocs.pm/elixir/1.12/Port.html#module-zombie-operating-system-processes).\n  > For example, in a Dockerfile you would add:\n  > `RUN apk add --no-cache bash libc6-compat`\n\n  * `:version` - The version of GrafanaAgent that you want to run. This is a string denoting the\n    GrafanaAgent release version. Below are the supported versions (the downloaded artifacts\n    are validated against their known SHA256 values so that you can be sure you are not downloading\n    any malicious binaries and running them). By default, PromEx will use the result of\n    `PromEx.GrafanaAgent.Downloader.default_version()` if no value is provided.\n\n    * Supported versions are `[\"0.42.0\"]`\n\n  * `:working_directory` - In order to leverage the GrafanaAgent functionality, PromEx needs to have\n    read/write access to a directory in order to download and copy the GrafanaAgent binary. This is the\n    full path to that directory.\n\n  * `:config_opts` - The configuration file that GrafanaAgent is started with. This option\n    can either accept an MFA that will return a string of the full path where the YAML configuration\n    file is, or a keyword list with options so that PromEx can generate a config file for you. If you\n    take the route where PromEx generates a config file for you, you must provide the following\n    options:\n\n    * `:metrics_server_path` - The path where the Prometheus metrics are exposed.\n\n    * `:metrics_server_port` - The port that the metrics server is running on.\n\n    * `:metrics_server_scheme` - Whether the app reachable via HTTPS or HTTP (default is `https`).\n\n    * `:metrics_server_host` - The host to scrape for metrics.\n\n    * `:instance` - This value denotes what instance the metrics are associated with. This value\n      is a string and defaults to the hostname.\n\n    * `:job` - This value denotes what job the metrics are associated with. This value\n      is a string and defaults to the otp_app.\n\n    * `:agent_port` - What port should GrafanaAgent run on.\n\n    * `:grpc_port` - What port should GrafanaAgent gRPC server run on.\n\n    * `:scrape_interval` - How often should GrafanaAgent scrape the application. The default is `15s`.\n\n    * `:bearer_token` - The bearer token that GrafanaAgent should attach to the request to your app.\n\n    * `:log_level` - The logging level for GrafanaAgent.\n\n    * `:prometheus_url` - The url to your Prometheus instance.\n\n    * `:prometheus_username` - The username to the hosted Prometheus instance\n\n    * `:prometheus_password` - The password to the hosted Prometheus instance\n\n    * `:template_file` - The full path to the template used to render the agent config file.\n\n    * all of these keys, and any additional ones, will be provided as EEx vars to the template file.\n\n* `:metrics_server` - This key contains the configuration information needed to run a standalone\n  HTTP server powered by Cowboy. This server provides a lightweight solution to serving up PromEx\n  metrics. In order to use this standalone metrics server plug you need to have `:plug` and `:plug_cowboy`\n  as dependencies in your project. Its configuration options are:\n\n  * `:port` - The port that the Cowboy HTTP server should run on.\n\n  * `:path` - The path that the metrics should be accessible at.\n\n  * `:protocol` - The protocol that the metrics should be accessible over (`:http` or `:https`).\n\n  * `:pool_size` - How many Cowboy processes should be in the pool to handle metrics related requests.\n\n  * `:auth_strategy` - What authentication strategy should be used to authorize requests to your metrics. The\n    Supported strategies are `:none`, `:bearer`, and `:basic`. Depending on what strategy is selected, you\n    will need to also add additional config values. For `:none` (which is the default), no additional\n    information needs to be provided. When using a `:bearer` strategy, you'll need to provide a `:auth_token`\n    config value. When using `:basic` strategy you'll need to provide `:auth_user` and `:auth_password` values.\n\n  * `:auth_token` - When using a `:bearer` authentication strategy, this field is required to validate the\n    incoming request against a valid auth token.\n\n  * `:auth_user` - When using a `:basic` authentication strategy, this field is required to validate the\n    incoming request against a valid user.\n\n  * `:auth_password` - When using a `:bearer` authentication strategy, this field is required to validate the\n    incoming request against a valid password.\n\n  * `:cowboy_opts` - A keyword list of any additional options that should be passed to `Plug.Cowboy` (see\n    docs for more information https://hexdocs.pm/plug_cowboy/Plug.Cowboy.html). The `:port` and\n    `:transport_options` options are handled by PromEx via the aforementioned config settings and so\n    adding them again here has no effect.","ref":"PromEx.Config.html#module-environment-dependencies-warning"},{"type":"function","title":"PromEx.Config.build/1","doc":"Create a struct that encapsulates all of the configuration needed to start a PromEx supervisor instance as well as all\nof the worker processes.","ref":"PromEx.Config.html#build/1"},{"type":"type","title":"PromEx.Config.t/0","doc":"- `disabled`: Whether PromEx will start up the metric collection supervision tree.\n- `manual_metrics_start_delay`: How the ManualMetricsManager worker process should be started (instantly or with a millisecond delay).\n- `drop_metrics_groups`: A list of metrics groups that should be omitted from the metrics collection process.\n- `ets_flush_interval`: How often should the ETS buffer table be compacted.\n- `grafana_config`: A map containing all the relevant settings to connect to Grafana.\n- `grafana_agent_config`: A map containing all the relevant settings to connect to GrafanaAgent.\n- `metrics_server_config`: A map containing all the relevant settings to start a standalone HTTP Cowboy server for metrics.","ref":"PromEx.Config.html#t:t/0"},{"type":"module","title":"PromEx.DashboardRenderer","doc":"This module is used to read dashboard definitions, render EEx dashboards,\nand ensure that requested files actually exist","ref":"PromEx.DashboardRenderer.html"},{"type":"function","title":"PromEx.DashboardRenderer.apply_dashboard_function/2","doc":"This function will execute a provided function on the rendered dashboard to give the user a chance to\nadjust the dashboard definition.","ref":"PromEx.DashboardRenderer.html#apply_dashboard_function/2"},{"type":"function","title":"PromEx.DashboardRenderer.build/3","doc":"Read the contents of a dashboard file entry. If an EEx version of the file exists,\nread that, else fallback to the provided file name. For example, if the requested\ndashboard file is `ecto.json`, if `ecto.json.eex` exists, then that will be returned.\nThis is more so for convenience so that you don't need to write out `.eex` everywhere\ngiven that all PromEx 1st party dashboards are EEx templates.","ref":"PromEx.DashboardRenderer.html#build/3"},{"type":"function","title":"PromEx.DashboardRenderer.decode_dashboard/1","doc":"This function will decode the JSON dashboard using `Jason`. If any errors occur during the decoding process,\nthe struct will be marked as having invalid JSON.","ref":"PromEx.DashboardRenderer.html#decode_dashboard/1"},{"type":"function","title":"PromEx.DashboardRenderer.merge_assigns/2","doc":"This function will merge in the provided assigns to the struct's assigns. These assigns will\nthen be used during the `render_dashboard/1` call to render any EEx template statements.","ref":"PromEx.DashboardRenderer.html#merge_assigns/2"},{"type":"function","title":"PromEx.DashboardRenderer.render_dashboard/2","doc":"Renders the dashboard. If it is an EEx file then the PromEx module assigns are passed. Else\nif it is a raw json file then it is passed through untouched.","ref":"PromEx.DashboardRenderer.html#render_dashboard/2"},{"type":"type","title":"PromEx.DashboardRenderer.t/0","doc":"","ref":"PromEx.DashboardRenderer.html#t:t/0"},{"type":"module","title":"PromEx.DashboardUploader","doc":"This GenServer is responsible for uploading the configured PromEx module\ndashboards to Grafana. This is a transient process and will terminate after\nthe dashboards have been successfully uploaded. It requires the name of the\nPromEx module as an option so that it can look into the application\nconfig for the appropriate Grafana settings. For example, if the name of the\nPromEx module is `WebApp.PromEx`, then your config should provide the following\nsettings:\n\n```elixir\nconfig :web_app, WebApp.PromEx,\n  grafana_host: \" \",\n  grafana_auth_token: \" \"\n```","ref":"PromEx.DashboardUploader.html"},{"type":"function","title":"PromEx.DashboardUploader.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PromEx.DashboardUploader.html#child_spec/1"},{"type":"function","title":"PromEx.DashboardUploader.start_link/1","doc":"Used to start the DashboardUploader process","ref":"PromEx.DashboardUploader.html#start_link/1"},{"type":"module","title":"PromEx.Debug","doc":"This is a convenience module used for debugging and introspecting\ntelemetry events. Primarily used to ease the development of\nPromEx itself.","ref":"PromEx.Debug.html"},{"type":"function","title":"PromEx.Debug.attach_debugger/1","doc":"Use this function to attach a debugger handler to a certain telemetry event.","ref":"PromEx.Debug.html#attach_debugger/1"},{"type":"module","title":"PromEx.ETSCronFlusher","doc":"This module is used to regularly flush ETS of any buffered distribution\ntype metrics (see https://github.com/beam-telemetry/telemetry_metrics_prometheus_core/blob/main/lib/core.ex#L25-L28)\nfor more information. At the moment the flush interval is not configurable\nbut that could change in the future.","ref":"PromEx.ETSCronFlusher.html"},{"type":"function","title":"PromEx.ETSCronFlusher.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PromEx.ETSCronFlusher.html#child_spec/1"},{"type":"function","title":"PromEx.ETSCronFlusher.defer_ets_flush/1","doc":"This function will cancel the existing cron timer and schedule a new\nnew. The reason for this being that if metrics scrapes occur regularly,\nthere is no need to flush via the cron job. Cron ETS flushes should\nonly occur if the metrics endpoint for whatever reason fails to be\nscraped so that ETS is not leaking memory.","ref":"PromEx.ETSCronFlusher.html#defer_ets_flush/1"},{"type":"function","title":"PromEx.ETSCronFlusher.start_link/1","doc":"Used to start the `PromEx.ETSCronFlusher` process.","ref":"PromEx.ETSCronFlusher.html#start_link/1"},{"type":"module","title":"PromEx.GrafanaAgent","doc":"This GenServer is responsible for starting the Grafana Agent\nbinary via a port and ensuring that it stays up and running.","ref":"PromEx.GrafanaAgent.html"},{"type":"function","title":"PromEx.GrafanaAgent.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PromEx.GrafanaAgent.html#child_spec/1"},{"type":"function","title":"PromEx.GrafanaAgent.start_link/1","doc":"Start the GrafanaAgent GenServer","ref":"PromEx.GrafanaAgent.html#start_link/1"},{"type":"module","title":"PromEx.GrafanaAgent.ConfigRenderer","doc":"This module is used to render the YAML configuration file for\nGrafanaAgent.","ref":"PromEx.GrafanaAgent.ConfigRenderer.html"},{"type":"function","title":"PromEx.GrafanaAgent.ConfigRenderer.generate_config_file/2","doc":"Generate and write the GrafanaAgent config file.","ref":"PromEx.GrafanaAgent.ConfigRenderer.html#generate_config_file/2"},{"type":"module","title":"PromEx.GrafanaAgent.Downloader","doc":"This module is responsible for downloading the GrafanaAgent binary.","ref":"PromEx.GrafanaAgent.Downloader.html"},{"type":"module","title":"PromEx.GrafanaClient","doc":"This module is used by the Mix tasks that are available in PromEx to update\ndashboards in Grafana and also by the `PromEx.DashboardUpdater` to update\ndashboards automatically on application initialization.\n\nDashboard models:\nhttps://grafana.com/docs/grafana/latest/dashboards/json-model/","ref":"PromEx.GrafanaClient.html"},{"type":"function","title":"PromEx.GrafanaClient.build_conn/1","doc":"Build a grafana client connection.","ref":"PromEx.GrafanaClient.html#build_conn/1"},{"type":"function","title":"PromEx.GrafanaClient.create_annotation/4","doc":"Used to create annotations on dashboard panels.","ref":"PromEx.GrafanaClient.html#create_annotation/4"},{"type":"function","title":"Option Details - PromEx.GrafanaClient.create_annotation/4","doc":"* `:dashboard_id` - Create a dashboard specific annotation. Optional\n* `:panel_id` - Create a panel specific annotation. Optional\n* `:time` - Specify the time of the annotation. When non is supplied the annotation is inferred by grafana. Optional\n* `:time_end` - Specify to great a region annotation. Optional","ref":"PromEx.GrafanaClient.html#create_annotation/4-option-details"},{"type":"function","title":"PromEx.GrafanaClient.create_folder/3","doc":"Used to create a new folder in Grafana","ref":"PromEx.GrafanaClient.html#create_folder/3"},{"type":"function","title":"PromEx.GrafanaClient.get_all_folders/1","doc":"Get all of the folder that exist in Grafana","ref":"PromEx.GrafanaClient.html#get_all_folders/1"},{"type":"function","title":"PromEx.GrafanaClient.get_dashboard/2","doc":"Used to get the dashboard definition currently in Grafana for the provided dashboard file.\nIf the ID does not exist in Grafana an error tuple will be returned.","ref":"PromEx.GrafanaClient.html#get_dashboard/2"},{"type":"function","title":"PromEx.GrafanaClient.get_folder/2","doc":"Used to fetch the details regarding a particular folder on Grafana","ref":"PromEx.GrafanaClient.html#get_folder/2"},{"type":"function","title":"PromEx.GrafanaClient.update_folder/4","doc":"Update an existing folder in Grafana","ref":"PromEx.GrafanaClient.html#update_folder/4"},{"type":"function","title":"PromEx.GrafanaClient.upload_dashboard/3","doc":"Used to create a new dashboard or update an existing dashboard.","ref":"PromEx.GrafanaClient.html#upload_dashboard/3"},{"type":"module","title":"PromEx.GrafanaClient.Connection","doc":"This struct encapsulates all of the data necessary\nto connect to a Grafana instance.","ref":"PromEx.GrafanaClient.Connection.html"},{"type":"function","title":"PromEx.GrafanaClient.Connection.build/2","doc":"Build a connection struct for connecting to Grafana.","ref":"PromEx.GrafanaClient.Connection.html#build/2"},{"type":"type","title":"PromEx.GrafanaClient.Connection.t/0","doc":"","ref":"PromEx.GrafanaClient.Connection.html#t:t/0"},{"type":"module","title":"PromEx.GrafanaClient.DashboardChecker","doc":"This module is used to validate Grafana dashboard to ensure that\nthey adhere to certain style and structure requirements.","ref":"PromEx.GrafanaClient.DashboardChecker.html"},{"type":"module","title":"PromEx.LifecycleAnnotator","doc":"This GenServer is responsible to keeping track of the life cycle\nof the application and sending annotation requests to Grafana\nwhen the application starts and when it terminates. It will\ninclude things in the message like:\n\n  - Hostname\n  - OTP app name\n  - App version\n  - Git SHA of the last commit (if the GIT_SHA environment variable is present)\n  - Git author of the last commit (if the GIT_AUTHOR environment variable is present)","ref":"PromEx.LifecycleAnnotator.html"},{"type":"function","title":"PromEx.LifecycleAnnotator.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PromEx.LifecycleAnnotator.html#child_spec/1"},{"type":"function","title":"PromEx.LifecycleAnnotator.start_link/1","doc":"Used to start the `PromEx.LifecycleAnnotator` process.","ref":"PromEx.LifecycleAnnotator.html#start_link/1"},{"type":"module","title":"PromEx.ManualMetricsManager","doc":"This GenServer is responsible to keeping track of all the manual\nmetrics from your configured plugins. It will fetch metrics initially\nwhen starting (either immediately or after a configured delay). At that\npoint if you would like to refresh your metrics data points call the\n`refresh_metrics/1` function.","ref":"PromEx.ManualMetricsManager.html"},{"type":"function","title":"PromEx.ManualMetricsManager.child_spec/1","doc":"Returns a specification to start this module under a supervisor.\n\nSee `Supervisor`.","ref":"PromEx.ManualMetricsManager.html#child_spec/1"},{"type":"function","title":"PromEx.ManualMetricsManager.refresh_metrics/1","doc":"This function calls into the GenServer and refreshes all of the configured\nmanual metrics data points.","ref":"PromEx.ManualMetricsManager.html#refresh_metrics/1"},{"type":"function","title":"PromEx.ManualMetricsManager.start_link/1","doc":"Used to start the `PromEx.ManualMetricsManager` process","ref":"PromEx.ManualMetricsManager.html#start_link/1"},{"type":"module","title":"PromEx.MetricTypes.Event","doc":"This struct defines the fields necessary to export a group of\nstandard metrics from a plugin.","ref":"PromEx.MetricTypes.Event.html"},{"type":"function","title":"PromEx.MetricTypes.Event.build/2","doc":"Create a struct that encompasses a group of event based metrics. The `group_name` should be unique and should follow\nthe following convention: ` _ _event_metrics`. For example, Phoenix HTTP related metrics\nhave a `group_name` of: `:phoenix_http_event_metrics`","ref":"PromEx.MetricTypes.Event.html#build/2"},{"type":"type","title":"PromEx.MetricTypes.Event.t/0","doc":"- `group_name`: A unique identifier for the collection of metrics.\n- `metrics`: A list of Telemetry Metrics structs that define the metrics.","ref":"PromEx.MetricTypes.Event.html#t:t/0"},{"type":"module","title":"PromEx.MetricTypes.Manual","doc":"This struct defines the fields necessary to export a group\nof manually collected metric sources from a plugin.","ref":"PromEx.MetricTypes.Manual.html"},{"type":"function","title":"PromEx.MetricTypes.Manual.build/3","doc":"Create a struct that encompasses a group of manually triggered metrics. The `group_name` should be unique and should follow\nthe following convention: ` _ _manual_metrics`. For example, Application related info metrics\nhave a `group_name` of: `:application_versions_manual_metrics`","ref":"PromEx.MetricTypes.Manual.html#build/3"},{"type":"type","title":"PromEx.MetricTypes.Manual.t/0","doc":"- `group_name`: A unique identifier for the collection of metrics.\n- `measurements_mfa`: An MFA tuple that defines what function will be\n  executed that will emit Telemetry events.\n- `metrics`: A list of Telemetry Metrics structs that define the metrics.","ref":"PromEx.MetricTypes.Manual.html#t:t/0"},{"type":"module","title":"PromEx.MetricTypes.Polling","doc":"This struct defines the fields necessary to export a group\nof pollable metric sources from a plugin.","ref":"PromEx.MetricTypes.Polling.html"},{"type":"function","title":"PromEx.MetricTypes.Polling.build/5","doc":"Create a struct that encompasses a group of polling type metrics. The `group_name` should be unique and should follow\nthe following convention: ` _ _manual_metrics`. For example, BEAM related memory metrics\nhave a `group_name` of: `:beam_memory_polling_events`","ref":"PromEx.MetricTypes.Polling.html#build/5"},{"type":"type","title":"PromEx.MetricTypes.Polling.t/0","doc":"- `group_name`: A unique identifier for the collection of metrics.\n- `measurements_mfa`: An MFA tuple that defines what function will be\n  executed that will emit Telemetry events.\n- `metrics`: A list of Telemetry Metrics structs that define the metrics.\n- `poll_rate`: An integer representing the millisecond between metrics samples.\n- `opts`: A keyword list of options to configure the execution of the polling job.\n  The supported options are currently:\n    - `detach_on_error`: Configures whether your MFA will stop being invoked\n      after it encounters an error. The default for this is `true`.","ref":"PromEx.MetricTypes.Polling.html#t:t/0"},{"type":"module","title":"PromEx.MetricsServer.Plug","doc":"This plug is used to serve metrics when PromEx is run in a\nstandalone server configuration. This plug supports the\nfollowing options:\n\n* `:path` - The route that should expose the metrics (default is \"/metrics\")\n* `:prom_ex_module` - The PromEx module that this plug will expose\n\nIn order to use this standalone metrics server plug you need to have `:plug` and `:plug_cowboy`\nas dependencies in your project.","ref":"PromEx.MetricsServer.Plug.html"},{"type":"module","title":"PromEx.Plug","doc":"Use this plug in your Endpoint file to expose your metrics. The following options are supported by this plug:\n\n* `:prom_ex_module` - The PromEx module whose metrics will be published through this particular plug\n* `:path` - The path through which your metrics can be accessed (default is \"/metrics\")\n\nIf you need to have some sort of access control around your metrics endpoint, I would suggest looking at another\nlibrary that I maintain called `Unplug` (https://hex.pm/packages/unplug). Using `Unplug`, you can skip over this plug\nif some sort of requirement is not fulfilled. For example, if you wanted to configure the metrics endpoint to\nonly be accessible if the request has an Authorization header that matches a configured environment variable you\ncould do something like so using `Unplug`:\n\n```elixir\ndefmodule MyApp.UnplugPredicates.SecureMetricsEndpoint do\n  @behaviour Unplug.Predicate\n\n  @impl true\n  def call(conn, env_var) do\n    expected_secret = System.fetch_env!(env_var)\n    match?([^expected_secret], Plug.Conn.get_req_header(conn, \"authorization\"))\n  end\nend\n```\n\nWhich can then be used in your `endpoint.ex` file like so:\n\n```elixir\nplug Unplug,\n  if: {MyApp.UnplugPredicates.SecureMetricsEndpoint, \"PROMETHEUS_AUTH_SECRET\"},\n  do: {PromEx.Plug, prom_ex_module: MyApp.PromEx}\n```\n\nThe reason that this functionality is not part of PromEx itself is that how you chose to configure the visibility\nof the metrics route is entirely up to the user and so it felt as though this plug would be over complicated by\nhaving to support application config, environment variables, etc. And given that `Unplug` exists for this purpose,\nit is the recommended tool for the job.","ref":"PromEx.Plug.html"},{"type":"behaviour","title":"PromEx.Plugin","doc":"This module defines the behaviour that PromEx plugins need to implement\nin order to be properly loaded by PromEx on application start. As a convenience, this\nmodule can also be used as a macro to automatically import all of the necessary utility\nfunctions for writing plugins and also providing default implementations of behaviour\nfunctions that you may not be implementing.","ref":"PromEx.Plugin.html"},{"type":"callback","title":"PromEx.Plugin.event_metrics/1","doc":"The `c:event_metrics/1` callback returns the configured event based metrics that the\nplugin exposes. If the plugin does not expose any event style metrics, there is a\ndefault implementation of this function that returns an empty list. In other words,\nif your plugin does not expose any event style metrics, there is no action needed\non your part.\n\nThis function is expected to either return a single `PromEx.Plugins.Event` struct\nor a list of `PromEx.Plugins.Event` structs.","ref":"PromEx.Plugin.html#c:event_metrics/1"},{"type":"callback","title":"PromEx.Plugin.manual_metrics/1","doc":"The `c:manual_metrics/1` callback returns the configured manual based metrics that the\nplugin exposes. If the plugin does not expose any manual style metrics, there is a\ndefault implementation of this function that returns an empty list. In other words,\nif your plugin does not expose any manual style metrics, there is no action needed\non your part.\n\nThis function is expected to either return a single `PromEx.Plugins.Manual` struct\nor a list of `PromEx.Plugins.Manual` structs.","ref":"PromEx.Plugin.html#c:manual_metrics/1"},{"type":"callback","title":"PromEx.Plugin.polling_metrics/1","doc":"The `c:polling_metrics/1` callback returns the configured polling based metrics that the\nplugin exposes. If the plugin does not expose any polling style metrics, there is a\ndefault implementation of this function that returns an empty list. In other words,\nif your plugin does not expose any polling style metrics, there is no action needed\non your part.\n\nThis function is expected to either return a single `PromEx.Plugins.Polling` struct\nor a list of `PromEx.Plugins.Polling` structs.","ref":"PromEx.Plugin.html#c:polling_metrics/1"},{"type":"module","title":"PromEx.Plugins.Absinthe","doc":"This plugin captures metrics emitted by Absinthe. Specifically, it captures timings and metrics\naround execution times, query complexity, and subscription timings. In order to get complexity\nmetrics you'll need to make sure that you have `:analyze_complexity` enabled in\n[Absinthe.Plug](https://hexdocs.pm/absinthe_plug/Absinthe.Plug.html#t:opts/0). This plugin can\ngenerate a large amount of Prometheus series, so it is suggested that you use the\n`ignored_entrypoints` and `only_entrypoints` (TODO: coming soon) options to prune down the\nresulting metrics if needed.\n\nThis plugin supports the following options:\n- `ignored_entrypoints`: This option is OPTIONAL and is used to filter out Absinthe GraphQL\n  schema entrypoints that you do not want to track metrics for. For example, if you don't want\n  metrics on the `:__schema` entrypoint (used for GraphQL schema introspection), you would set\n  a value of `[:__schema]`. This is applicable to queries, mutations, and subscriptions.\n\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :absinthe]`. If this changes you will also want to set `absinthe_metric_prefix`\n  in your `dashboard_assigns` to the snakecase version of your prefix, the default\n  `absinthe_metric_prefix` is `{otp_app}_prom_ex_absinthe`.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nThis plugin exposes the following metric groups:\n- `:absinthe_execute_event_metrics`\n- `:absinthe_subscription_event_metrics`\n\nTo use plugin in your application, add the following to your PromEx module:\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {PromEx.Plugins.Absinthe, ignored_entrypoints: [:__schema]}\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"absinthe.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.Absinthe.html"},{"type":"module","title":"PromEx.Plugins.Application","doc":"This plugin captures metrics regarding your application, and its dependencies. Specifically,\nit captures the versions of your application and the application dependencies and also\nhow many modules each dependency is bringing into the project.\n\nThis plugin supports the following options:\n- `otp_app`: This is a REQUIRED option and is the name of you application in snake case (e.g. :my_cool_app).\n\n- `deps`: This option is OPTIONAL and defines what dependencies the plugin should track. A value of `:all`\n  means that PromEx will fetch details on all application dependencies. A list of dependency names like\n  `[:phoenix, :ecto, :unplug]` means that PromEx will only fetch details regarding those dependencies.\n\n- `git_sha_mfa`: This option is OPTIONAL and defines an MFA that will be called in order to fetch the\n  application's Git SHA at the time of deployment. By default, an Application Plugin function will be called\n  and will attempt to read the GIT_SHA environment variable to populate the value.\n\n- `git_author_mfa`: This option is OPTIONAL and defines an MFA that will be called in order to fetch the\n  application's last Git commit author at the time of deployment. By default, an Application Plugin function\n  will be called and will attempt to read the GIT_AUTHOR environment variable to populate the value.\n\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :application]`. If this changes you will also want to set `application_metric_prefix`\n  in your `dashboard_assigns` to the snakecase version of your prefix, the default\n  `application_metric_prefix` is `{otp_app}_prom_ex_application`.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nThis plugin exposes the following metric groups:\n- `:application_versions_manual_metrics`\n\nTo use plugin in your application, add the following to your application supervision tree:\n```\ndef start(_type, _args) do\n  children = [\n    ...\n    {\n      PromEx,\n      plugins: [\n        {PromEx.Plugins.Application, [otp_app: :my_cool_app]},\n        ...\n      ],\n      delay_manual_start: :no_delay\n    }\n  ]\n\n  opts = [strategy: :one_for_one, name: WebApp.Supervisor]\n  Supervisor.start_link(children, opts)\nend\n```\n\nThis plugin exposes manual metrics so be sure to configure the PromEx `:delay_manual_start` as required.","ref":"PromEx.Plugins.Application.html"},{"type":"module","title":"PromEx.Plugins.Beam","doc":"Telemetry metrics for the BEAM.\n\nThis plugin captures metrics regarding the Erlang Virtual Machine (i.e the BEAM). Specifically, it captures metrics\nregarding the CPU topology, system limits, VM feature support, scheduler information, memory utilization, distribution\ntraffic, and other internal metrics.\n\nThis plugin supports the following options:\n- `poll_rate`: This is option is OPTIONAL and is the rate at which poll metrics are refreshed (default is 5 seconds).\n\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :beam]`. If this changes you will also want to set `beam_metric_prefix`\n  in your `dashboard_assigns` to the snakecase version of your prefix, the default\n  `beam_metric_prefix` is `{otp_app}_prom_ex_beam`.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nThis plugin exposes the following metric groups:\n- `:beam_memory_polling_metrics`\n- `:beam_internal_polling_metrics`\n- `:beam_cpu_topology_manual_metrics`\n- `:beam_system_limits_manual_metrics`\n- `:beam_system_info_manual_metrics`\n- `:beam_scheduler_manual_metrics`\n\nTo use plugin in your application, add the following to your PromEx module:\n```\ndefmodule MyApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      PromEx.Plugins.Beam\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"beam.json\"}\n    ]\n  end\nend\n```\n\nThis plugin exposes manual metrics so be sure to configure the PromEx `:delay_manual_start` as necessary.","ref":"PromEx.Plugins.Beam.html"},{"type":"module","title":"PromEx.Plugins.Broadway","doc":"This plugin captures metrics emitted by Broadway.\n\nThis plugin exposes the following metric groups:\n- `:broadway_init_event_metrics`\n- `:broadway_message_event_metrics`\n- `:broadway_batch_event_metrics`\n\nTo use plugin in your application, add the following to your PromEx module:\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      PromEx.Plugins.Broadway\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"broadway.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.Broadway.html"},{"type":"module","title":"PromEx.Plugins.Ecto","doc":"This plugin captures metrics emitted by Ecto. Be sure that your PromEx module is listed before your Repo module\nin your supervision tree so that the Ecto init events are not missed. If you miss those events the dashboard\nvariable dropdowns for the repo value will be broken.\n\nThis plugin supports the following options:\n- `otp_app`: This is an OPTIONAL option and is the name of you application in snake case (e.g. :my_cool_app). By\n  default the otp_app set for the prom_ex module that this plugin is defined in is used.\n\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :ecto]`. If this changes you will also want to set `ecto_metric_prefix`\n  in your `dashboard_assigns` to the snakecase version of your prefix, the default\n  `ecto_metric_prefix` is `{otp_app}_prom_ex_ecto`.\n\n- `repos`: This is an OPTIONAL option and is a list with the full module name of your Ecto Repos (e.g [MyApp.Repo]).\n   If you do not provide this value, PromEx will attempt to resolve your Repo modules via the\n   `:ecto_repos` configuration on your OTP app.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nThis plugin exposes the following metric groups:\n- `:ecto_init_event_metrics`\n- `:ecto_query_event_metrics`\n\nTo use plugin in your application, add the following to your PromEx module `plugins/0` function:\n```\ndef plugins do\n  [\n    ...\n\n    {PromEx.Plugins.Ecto, otp_app: :web_app, repos: [WebApp.Repo]}\n  ]\nend\n\n```","ref":"PromEx.Plugins.Ecto.html"},{"type":"module","title":"PromEx.Plugins.Oban","doc":"This plugin captures metrics emitted by Oban. Specifically, it captures metrics from job events, producer events,\nand also from internal polling jobs to monitor queue sizes\n\nThis plugin supports the following options:\n- `oban_supervisors`: This is an OPTIONAL option and it allows you to specify what Oban instances should have their events\n  tracked. By default the only Oban instance that will have its events tracked is the default `Oban` instance. As a result, by\n  default this option has a value of `[Oban]`. If you would like to track other named Oban instances, or perhaps your default\n  and only Oban instance has a different name, you can pass in your own list of Oban instances (e.g. `[Oban, Oban.PrivateJobs]`).\n\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :oban]`. If this changes you will also want to set `oban_metric_prefix`\n  in your `dashboard_assigns` to the snakecase version of your prefix, the default\n  `oban_metric_prefix` is `{otp_app}_prom_ex_oban`.\n\n- `poll_rate`: This option is OPTIONAL and is the rate at which poll metrics are refreshed (default is 5 seconds).\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nThis plugin exposes the following metric groups:\n- `:oban_init_event_metrics`\n- `:oban_job_event_metrics`\n- `:oban_producer_event_metrics`\n- `:oban_circuit_event_metrics`\n- `:oban_queue_poll_metrics`\n\nTo use plugin in your application, add the following to your PromEx module:\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {PromEx.Plugins.Oban, oban_supervisors: [Oban, Oban.AnotherSupervisor], poll_rate: 10_000}\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"oban.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.Oban.html"},{"type":"function","title":"PromEx.Plugins.Oban.oban_circuit_breaker_event_metrics/2","doc":"","ref":"PromEx.Plugins.Oban.html#oban_circuit_breaker_event_metrics/2"},{"type":"module","title":"PromEx.Plugins.Phoenix","doc":"This plugin captures metrics emitted by Phoenix. Specifically, it captures HTTP request metrics and\nPhoenix channel metrics.","ref":"PromEx.Plugins.Phoenix.html"},{"type":"module","title":"Plugin options - PromEx.Plugins.Phoenix","doc":"This plugin supports the following options:\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :phoenix]`. If this changes you will also want to set `phoenix_metric_prefix`\n  in your `dashboard_assigns` to the snakecase version of your prefix, the default\n  `phoenix_metric_prefix` is `{otp_app}_prom_ex_phoenix`.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.","ref":"PromEx.Plugins.Phoenix.html#module-plugin-options"},{"type":"module","title":"Single Endpoint/Router - PromEx.Plugins.Phoenix","doc":"- `router`: This option is REQUIRED and is the full module name of your Phoenix Router (e.g MyAppWeb.Router).\n\n- `endpoint`: This is a REQUIRED option and is the full module name of your Phoenix Endpoint (e.g MyAppWeb.Endpoint).\n\n- `event_prefix`: This option is OPTIONAL and allows you to set the event prefix for the Telemetry events. This\n  value should align with what you pass to `Plug.Telemetry` in your `endpoint.ex` file (see the plug docs\n  for more information https://hexdocs.pm/plug/Plug.Telemetry.html) This value should align with what you pass\n  to `Plug.Telemetry` in your `endpoint.ex` file (see the plug docs for more\n  information https://hexdocs.pm/plug/Plug.Telemetry.html)\n\n- `additional_routes`: This option is OPTIONAL and allows you to specify route path labels for applications routes\n  not defined in your Router module.\n\n  For example, if you want to track telemetry events for a plug in your\n  `endpoint.ex` file, you can provide a keyword list with the structure `[some-route: ~r(/some-path)]` and any\n  time that the route is called and the plug handles the call, the path label for this particular Prometheus metric\n  will be set to `some-route`. You can pass in either a regular expression or a string to match the incoming\n  request.\n\n- `normalize_event_name`: This option is OPTIONAL and allows you to remap the channel event names to a different\n  name. This is useful if you want to limit the number or size of event names that are emitted.\n\n  For example, if you have a channel and only want to emit unique metrics for the 'join' and 'leave' events, you can\n  map those events to themselves and map all other events to an 'unknown' event.\n\n  ```elixir\n    fn\n      \"join\" -> \"join\"\n      \"leave\" -> \"leave\"\n      _ -> \"unknown\"\n    end\n  ```\n\n#### Example plugin configuration\n\n```elixir\n{\n  PromEx.Plugins.Phoenix,\n  endpoint: MyApp.Endpoint,\n  router: MyAppWeb.Public.Router,\n  event_prefix: [:admin, :endpoint],\n  normalize_event_name: fn event -> event end)\n}\n```","ref":"PromEx.Plugins.Phoenix.html#module-single-endpoint-router"},{"type":"module","title":"Multiple Endpoints/Router - PromEx.Plugins.Phoenix","doc":"- `endpoints`: This accepts a list of per Phoenix Endpoint options `{endpoint_name, endpoint_opts}`\n  - `endpoint_name`: This option is REQUIRED and is the full module name of your Phoenix Endpoint (e.g MyAppWeb.Endpoint).\n\n  - `endpoint_opts`: Per endpoint plugin options:\n    - `:routers`: This option is REQUIRED and lists all of routers modules for the endpoint, the HTTP metrics will\n      be augmented with controller/action/path information from the routers.\n\n    - `:event_prefix`: This option is OPTIONAL and allows you to set the event prefix for the Telemetry events. This\n    value should align with what you pass to `Plug.Telemetry` in the  corresponding endpoint module (see the plug docs\n    for more information https://hexdocs.pm/plug/Plug.Telemetry.html)\n\n    - `:additional_routes`: This option is OPTIONAL and allows you to specify route path labels for applications routes\n    not defined in your Router modules for the corresponding endpoint.\n\n#### Example plugin configuration\n\n```elixir\n{\n  PromEx.Plugins.Phoenix,\n  endpoints: [\n    {MyApp.Endpoint, routers: [MyAppWeb.Public.Router]},\n    {MyApp.Endpoint2, routers: [MyAppWeb.Admin.Router], event_prefix: [:admin, :endpoint]}\n  ]\n}\n```","ref":"PromEx.Plugins.Phoenix.html#module-multiple-endpoints-router"},{"type":"module","title":"Metric Groups - PromEx.Plugins.Phoenix","doc":"This plugin exposes the following metric groups:\n- `:phoenix_http_event_metrics`\n- `:phoenix_channel_event_metrics`\n- `:phoenix_socket_event_metrics`\n- `:phoenix_endpoint_metrics`","ref":"PromEx.Plugins.Phoenix.html#module-metric-groups"},{"type":"module","title":"Usage - PromEx.Plugins.Phoenix","doc":"To use plugin in your application, add the following to your PromEx module:\n\n```elixir\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {\n        PromEx.Plugins.Phoenix,\n        endpoint: MyApp.Endpoint,\n        router: MyAppWeb.Public.Router\n      }\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"phoenix.json\"}\n    ]\n  end\nend\n```\n\nWhen working with multiple Phoenix routers use the `endpoints` option instead:\n\n```elixir\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {\n        PromEx.Plugins.Phoenix,\n        endpoints: [\n          {MyApp.Endpoint, routers: [MyAppWeb.Public.Router]},\n          {MyApp.Endpoint2, routers: [MyAppWeb.Admin.Router], event_prefix: [:admin, :endpoint]}\n        ]\n      }\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"phoenix.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.Phoenix.html#module-usage"},{"type":"module","title":"PromEx.Plugins.PhoenixLiveView","doc":"This plugin captures metrics emitted by PhoenixLiveView. Specifically, it captures events related to the\nmount, handle_event, and handle_params callbacks for live views and live components.\n\nThis plugin supports the following options:\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n  `[otp_app, :prom_ex, :phoenix_live_view]`. If this changes you will also want to set\n  `phoenix_live_view_metric_prefix` in your `dashboard_assigns` to the snakecase version of your\n  prefix, the default `phoenix_live_view_metric_prefix` is `{otp_app}_prom_ex_phoenix_live_view`.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nThis plugin exposes the following metric groups:\n- `:phoenix_live_view_event_metrics`\n- `:phoenix_live_view_component_event_metrics`\n\nTo use plugin in your application, add the following to your PromEx module:\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      PromEx.Plugins.PhoenixLiveView\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"phoenix_live_view.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.PhoenixLiveView.html"},{"type":"module","title":"PromEx.Plugins.PlugCowboy","doc":"This plugin captures HTTP request metrics emitted by Plug.Cowboy.\n\nThis plugin exposes the following metric group:\n- `:plug_cowboy_http_event_metrics`","ref":"PromEx.Plugins.PlugCowboy.html"},{"type":"module","title":"Plugin options - PromEx.Plugins.PlugCowboy","doc":"- `routers`: **Required** This is a list with the full module names of your Routers (e.g MyAppWeb.Router).\n  Phoenix and Plug routers are supported. When the Phoenix dependency is present in your project, a list of Phoenix Routers is expected. Otherwise a list of Plug.Router modules must be provided\n- `event_prefix`: **Optional**, allows you to set the event prefix for the Telemetry events.\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n`[otp_app, :prom_ex, :plug_cowboy]`. If this changes you will also want to set `plug_cowboy_metric_prefix`\nin your `dashboard_assigns` to the snakecase version of your prefix, the default\n`plug_cowboy_metric_prefix` is `{otp_app}_prom_ex_plug_cowboy`.\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\nTo use plugin in your application, add the following to your PromEx module:\n\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {PromEx.Plugins.PlugCowboy, routers: [MyApp.Router]}\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"plug_cowboy.json\"}\n    ]\n  end\nend\n```\n\nTo ignore certain paths, pass a list of routes using the `:ignore_routes` option\n\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {PromEx.Plugins.PlugCowboy, routers: [MyApp.Router], ignore_routes: [\"/metrics\"]}\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"plug_cowboy.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.PlugCowboy.html#module-plugin-options"},{"type":"module","title":"PromEx.Plugins.PlugRouter","doc":"This plugin captures HTTP request metrics emitted by `Plug.Router` and `Plug.Telemetry`.\n\nThis plugin is heavily inspired on plugin `Plug.Cowboy`, and exposes the following metric group:\n - `:plug_router_http_event_metrics`","ref":"PromEx.Plugins.PlugRouter.html"},{"type":"module","title":"Plugin options - PromEx.Plugins.PlugRouter","doc":"- `routers`: **Required** This is a list with the full module names of your Routers (e.g MyAppWeb.Router).\n Metrics produced by routers not in this list will be discarded.\n\n- `event_prefix`: **Required**, allows you to set the event prefix defined in your `Plug.Telemetry` configuration:\n\n- `duration_unit`: This is an OPTIONAL option and is a `Telemetry.Metrics.time_unit()`. It can be one of:\n  `:second | :millisecond | :microsecond | :nanosecond`. It is `:millisecond` by default.\n\n```\ndefmodule WebApp.Router do\n  use Plug.Router\n\n  plug PromEx.Plug, prom_ex_module: WebApp.PromEx, path: \"/metrics\"\n  plug Plug.Telemetry, event_prefix: [:webapp, :router]\n  ...\nend\n```\n\nWith the above configuration, this plugin will subscribe to `[:webapp, :router, :stop]` telemetry events\nproduced by `Plug.Telemetry`. These events will be fired **before** the response is actually sent, therefore this\nplugin will be able to export response body size metrics, since the `Plug.Conn` struct in the metadata of the\ntelemetry measurement still contains the response body.\n\nHowever, `Plug.Telemetry` does not use `:telemetry.span/3`, which means the `:stop` event might not always be fired\n(eg. if the process handling the request crashes). For this reason, this PromEx plugin also subscribes to\n`[:plug, :router_dispatch, :exception]` telemetry events fired by `Plug.Router`, which are fired within a\n`:telemetry.span/3` call.\n\nUnfortunately, we cannot safely rely on `[:plug, :router_dispatch, :stop]` events produced by `Plug.Router` since\nthese are fired **after** the response is sent (as opposed to `Plug.Telemetry` `:stop` events). As a consequence,\nthe response body is no longer available in the `Plug.Conn` struct attached to the telemetry measurement metadata.\n\n- `metric_prefix`: This option is OPTIONAL and is used to override the default metric prefix of\n `[otp_app, :prom_ex, :plug_router]`. If this changes you will also want to set `plug_router_metric_prefix`\n in your `dashboard_assigns` to the snakecase version of your prefix, the default\n `plug_router_metric_prefix` is `{otp_app}_prom_ex_plug_router`.\n- `ignore_routes`: This option is OPTIONAL and is used to ignore certain paths.\n\nTo use plugin in your application, add the following to your PromEx module:\n\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  alias PromEx.Plugins\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {Plugins.PlugRouter,\n        event_prefix: [:webapp, :router], metric_prefix: [:prom_ex, :router], routers: [WebApp.Router]}\n    ]\n  end\n\n  @impl true\n  def dashboard_assigns do\n    [\n      datasource_id: \"...\",\n      plug_router_metric_prefix: \"prom_ex_router\"\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"plug_router.json\"}\n    ]\n  end\nend\n```\n\nTo ignore certain paths, pass a list of routes using the `:ignore_routes` option\n\n```\ndefmodule WebApp.PromEx do\n  use PromEx, otp_app: :web_app\n\n  @impl true\n  def plugins do\n    [\n      ...\n      {PromEx.Plugins.PlugRouter,\n        event_prefix: [:webapp, :router], metric_prefix: [:prom_ex, :router], routers: [WebApp.Router],\n        ignore_routes: [\"/metrics\"]}\n    ]\n  end\n\n  @impl true\n  def dashboards do\n    [\n      ...\n      {:prom_ex, \"plug_router.json\"}\n    ]\n  end\nend\n```","ref":"PromEx.Plugins.PlugRouter.html#module-plugin-options"},{"type":"behaviour","title":"PromEx.Storage","doc":"Storage definition behaviour.","ref":"PromEx.Storage.html"},{"type":"callback","title":"PromEx.Storage.child_spec/2","doc":"Define child specs for gatherer process.","ref":"PromEx.Storage.html#c:child_spec/2"},{"type":"function","title":"PromEx.Storage.child_spec/3","doc":"Generate the process child spec for the store.","ref":"PromEx.Storage.html#child_spec/3"},{"type":"callback","title":"PromEx.Storage.scrape/1","doc":"Gather metrics for given collector with given `name`.","ref":"PromEx.Storage.html#c:scrape/1"},{"type":"function","title":"PromEx.Storage.scrape/2","doc":"Extract the metrics out of the configured store.","ref":"PromEx.Storage.html#scrape/2"},{"type":"module","title":"PromEx.Storage.Core","doc":"This store uses [TelemetryMetricsPrometheus.Core](https://github.com/beam-telemetry/telemetry_metrics_prometheus_core) as the storage\nmechanism for metrics.","ref":"PromEx.Storage.Core.html"},{"type":"module","title":"PromEx.Storage.Peep","doc":"This store uses [Peep](https://github.com/rkallos/peep) as the storage\nmechanism for metrics.","ref":"PromEx.Storage.Peep.html"},{"type":"module","title":"PromEx.Utils","doc":"This module provides several general purpose utilities\nfor use in PromEx plugs.","ref":"PromEx.Utils.html"},{"type":"function","title":"PromEx.Utils.make_plural_atom/1","doc":"Converts a `time_unit` to its plural form.","ref":"PromEx.Utils.html#make_plural_atom/1"},{"type":"function","title":"PromEx.Utils.normalize_exception/3","doc":"Normalize exception messages for use as metric labels.","ref":"PromEx.Utils.html#normalize_exception/3"},{"type":"function","title":"PromEx.Utils.normalize_module_name/1","doc":"Take a module name and normalize it for use as a metric\nlabel.","ref":"PromEx.Utils.html#normalize_module_name/1"},{"type":"type","title":"PromEx.Utils.duration_unit_plural/0","doc":"","ref":"PromEx.Utils.html#t:duration_unit_plural/0"},{"type":"type","title":"PromEx.Utils.exception_kind/0","doc":"The kinds of exceptions that can occur","ref":"PromEx.Utils.html#t:exception_kind/0"},{"type":"extras","title":"README","doc":"Prometheus metrics and Grafana dashboards for all of your favorite Elixir libraries \n \n--------------------\n\n[![Hex.pm](https://img.shields.io/hexpm/v/prom_ex?style=for-the-badge)](http://hex.pm/packages/prom_ex)\n[![Build Status](https://img.shields.io/github/actions/workflow/status/akoutmos/prom_ex/main.yml?label=Build%20Status&style=for-the-badge&branch=master)](https://github.com/akoutmos/prom_ex/actions)\n[![Coverage Status](https://img.shields.io/coveralls/github/akoutmos/prom_ex/master?style=for-the-badge)](https://coveralls.io/github/akoutmos/prom_ex?branch=master)\n[![Elixir Slack Channel](https://img.shields.io/badge/slack-%23prom__ex-orange.svg?style=for-the-badge&logo=slack)](https://elixir-lang.slack.com/archives/C01NZ0FBFSR)\n[![Support PromEx](https://img.shields.io/badge/Support%20PromEx-%E2%9D%A4-lightblue?style=for-the-badge)](https://github.com/sponsors/akoutmos)\n\n\n# Contents\n\n- [Installation](#installation)\n- [Supporting PromEx](#supporting-promex)\n- [Setting Up PromEx](#setting-up-promex)\n- [Technical Support](#technical-support)\n- [Adding Your Metrics](#adding-your-metrics)\n- [Design Philosophy](#design-philosophy)\n- [Available Plugins](#available-plugins)\n- [Grafana Dashboards](#grafana-dashboards)\n- [Security Concerns](#security-concerns)\n- [Performance Concerns](#performance-concerns)\n- [Attribution](#attribution)","ref":"readme.html"},{"type":"extras","title":"Installation - README","doc":"[Available in Hex](https://hex.pm/packages/prom_ex), the package can be installed by adding `prom_ex` to your list of\ndependencies in `mix.exs`:\n\n```elixir\ndef deps do\n  [\n    {:prom_ex, \"~> 1.11.0\"}\n  ]\nend\n```\n\nDocumentation can be found at [https://hexdocs.pm/prom_ex](https://hexdocs.pm/prom_ex).","ref":"readme.html#installation"},{"type":"extras","title":"Supporting PromEx - README","doc":"PromEx has an ambition goal: To provide Prometheus metrics and Grafana dashboards for as many of the libraries in the Elixir ecosystem as possible as well as a framework so that you can manage the lifecycle of your own metrics and dashboards.\n\nIf you rely on PromEx to monitor your production applications, it would much appreciated if you can give back to the\nproject in order to help ensure its continued development.","ref":"readme.html#supporting-promex"},{"type":"extras","title":"Gold Sponsors - README","doc":"","ref":"readme.html#gold-sponsors"},{"type":"extras","title":"Silver Sponsors - README","doc":"","ref":"readme.html#silver-sponsors"},{"type":"extras","title":"Bronze Sponsors - README","doc":"","ref":"readme.html#bronze-sponsors"},{"type":"extras","title":"Setting Up PromEx - README","doc":"The goal of PromEx is to have metrics set up be as simple and streamlined as possible. In that spirit, all\nthat you need to do to start leveraging PromEx along with the built-in plugins is to run the following mix\ntask (the `YOUR_PROMETHEUS_DATASOURCE_ID` value should align with what is configured in Grafana as the name of the\nPrometheus data source):\n\n```\n$ mix prom_ex.gen.config --datasource YOUR_PROMETHEUS_DATASOURCE_ID\n```\n\nThen add the generated module to your `application.ex` file supervision tree (be sure to add it to the top of the\nsupervisor children list so that you do not miss any init-style events from other processes like Ecto.Repo for\nexample):\n\n```elixir\ndefmodule MyCoolApp.Application do\n  use Application\n\n  def start(_type, _args) do\n    children = [\n      # PromEx should be started before anything else as PromEx will caputre init events from\n      # libraries like Ecto, Phoenix and Oban. If it is started after those other supervision trees\n      # you will miss those events and metrics.\n      MyCoolApp.PromEx,\n\n      MyCoolApp.Repo,\n      MyCoolApp.Endpoint\n      ...\n    ]\n\n    opts = [strategy: :one_for_one, name: MyCoolApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\nend\n```\n\nWith that in place, all that you need to do is then add the PromEx plug somewhere in your\n`endpoint.ex` file (I would suggest putting it before your `plug Plug.Telemetry` call so that\nyou do not pollute your logs with calls to `/metrics`):\n\n```elixir\ndefmodule MyCoolAppWeb.Endpoint do\n  use Phoenix.Endpoint, otp_app: :my_cool_app\n\n  ...\n\n  plug PromEx.Plug, prom_ex_module: MyCoolApp.PromEx\n  # Or plug PromEx.Plug, path: \"/some/other/metrics/path\", prom_ex_module: MyCoolApp.PromEx\n\n  ...\n\n  plug Plug.RequestId\n  plug Plug.Telemetry, event_prefix: [:phoenix, :endpoint]\n\n  ...\n\n  plug MyCoolAppWeb.Router\nend\n```\n\nWith that in place, all you need to do is start your server and you should be able to hit your\nmetrics endpoint and see your application metrics:\n\n```terminal\n$ curl localhost:4000/metrics\n# HELP my_cool_app_application_dependency_info Information regarding the application's dependencies.\n# TYPE my_cool_app_application_dependency_info gauge\nmy_cool_app_application_dependency_info{modules=\"69\",name=\"hex\",version=\"0.20.5\"} 1\nmy_cool_app_application_dependency_info{modules=\"1\",name=\"connection\",version=\"1.0.4\"} 1\nmy_cool_app_application_dependency_info{modules=\"4\",name=\"telemetry_poller\",version=\"0.5.1\"} 1\n...\n```\n\nBe sure to check out the module docs for each plugin that you choose to use to ensure that you are familiar\nwith all of the options that they provide.","ref":"readme.html#setting-up-promex"},{"type":"extras","title":"Technical Support - README","doc":"Is your company using or going to use PromEx in production and you need some help? Perhaps you want to create\nsome custom Grafana dashboards and PromEx plugins specific to your business KPIs? What ever your observability\nneeds may be, I can definitely help you and your business out!\n\nFeel free to reach out to me via [Twitter](https://twitter.com/akoutmos), [Elixir Slack](https://elixir-lang.slack.com/archives/D60LPD0N8),\nor my consultancy `Contact Us` form:","ref":"readme.html#technical-support"},{"type":"extras","title":"Adding Your Metrics - README","doc":"While PromEx provides plugins for a lot of the popular community libraries, you can also leverage it to collect\nmetrics specific to your application. Similarly to how you use the PromEx Ecto plugin to collect metrics\nrelevant to Ecto, you can write your own plugin to collect metrics specific to your application. In fact, the same\ntools that PromEx uses for its plugins are what you should use to create your own application specific plugins.\n\nTo write your own PromEx plugin, create a module that implements the `PromEx.Plugin` behaviour and collect the\nrelevant event/polling/manual metrics. Be sure to check out the 1st party PromEx plugins as a reference for how\nplugins are written and how to collect the different types of data points (also checkout the [Writing PromEx Plugins\n](https://hexdocs.pm/prom_ex/writing-promex-plugins.html) guide).\n\nAs a side note, PromEx will attach its own Telemetry handlers to events in order to capture Prometheus compatible\nmetrics and so any data points that are added to your `telemetry.ex` file (if you are using LiveDashboard) will\nnot show up in PromEx. One of the benefits of the Telemetry library is that you can have an arbitrary number of\nevent handlers attached to Telemetry events and so LiveDashboard and PromEx can operate in the same application\nwithout any issues.","ref":"readme.html#adding-your-metrics"},{"type":"extras","title":"Design Philosophy - README","doc":"With the widespread adoption of the Telemetry library and the other libraries in the [BEAM Telemetry GitHub\nOrg](https://github.com/beam-telemetry), we have reached a point in the Elixir ecosystem where we have a consistent\nmeans of surfacing application and library metrics. This allows us to have a great level of insight into our\napplications and dependencies given that they all leverage the same fundamental tooling. The goal of this project is to\nprovide a \"Plug-in\" style library where you can easily add new plug-ins to surface metrics so that Prometheus can scrape\nthem. Ideally, this project acts as the \"Metrics\" pillar in your application (in reference to [The Three Pillars of\nObservability](https://www.oreilly.com/library/view/distributed-systems-observability/9781492033431/ch04.html)).\n\nTo this end, while PromEx does provide a certain level of configurability (like the polling rate, starting behaviour for\nmanual metrics and all the options that the plugins receive), the goal is not to make an infinitely configurable tool.\nFor example, you are not able to edit the names/descriptions of Prometheus metrics via plugin options or even the tags\nthat are attached to the data points.\n\nInstead, if there things that you don't agree with or that are incompatible with your usage of a certain 1st party\nplugin and want to edit how the PromEx plugins react to Telemetry events, it is recommended that you fork the plugin in\nquestion and edit it to your specific use case. If you think that the community can benefit for your changes, do not\nhesitate to make a PR and I'll be sure to review it. This is not to say that event configurability will never come to\nPromEx, but I want to make sure that the public facing API is clean and straightforward and not bogged down with too\nmuch configuration. In addition, the Grafana dashboards would then have to have a lot of templatized logic to\naccommodate all this configurability (something which has been a pain-point in the Helm community for example).\n\nPromEx provides the following utilities to you in order to achieve your observability goals:\n\n- The `PromEx.Plug` module that can be used in your Phoenix or Plug application to expose the collected metrics\n- A standalone HTTP metrics server if Phoenix is not a dependency in your project\n- A Mix task to upload the provided complimentary Grafana dashboards\n- A Mix task to create a PromEx metrics capture module\n- A behaviour that defines the contract for PromEx plug-ins\n- A behaviour that defines the functionality of a PromEx metrics capture module\n- Grafana dashboards tailored to each specific Plugin so that metrics work out of the box with dashboards\n- Grafana API support to create/upload to dashboard folders and to create graph annotations to mark events in Grafana\n- EEx Grafana dashboard templates so you can dynamically tweak dashboards prior to uploading","ref":"readme.html#design-philosophy"},{"type":"extras","title":"Available Plugins - README","doc":"| Plugin                           | Status      | Description                                                            |\n| -------------------------------- | ----------- | ---------------------------------------------------------------------- |\n| `PromEx.Plugins.Application`     | Stable      | Collect metrics on your application dependencies                       |\n| `PromEx.Plugins.Beam`            | Stable      | Collect metrics regarding the BEAM virtual machine                     |\n| `PromEx.Plugins.Phoenix`         | Stable      | Collect request metrics emitted by Phoenix                             |\n| `PromEx.Plugins.Ecto`            | Stable      | Collect query metrics emitted by Ecto                                  |\n| `PromEx.Plugins.Oban`            | Stable      | Collect queue processing metrics emitted by Oban                       |\n| `PromEx.Plugins.PhoenixLiveView` | Stable      | Collect metrics emitted by Phoenix LiveView                            |\n| `PromEx.Plugins.Absinthe`        | Stable      | Collect GraphQL metrics emitted by Absinthe                            |\n| `PromEx.Plugins.PlugCowboy`      | Stable      | Collect HTTP request metrics emitted by Plug.Cowboy                    |\n| `PromEx.Plugins.PlugRouter`      | Stable      | Collect HTTP request metrics emitted by Plug.Router and Plug.Telemetry |\n| `PromEx.Plugins.Broadway`        | Beta        | Collect message processing metrics emitted by Broadway                 |\n| `PromEx.Plugins.Finch`           | Coming soon | Collect HTTP request metrics emitted by Finch                          |\n| `PromEx.Plugins.Redix`           | Coming soon | Collect Redis request metrics emitted by Redix                         |\n| More to come...                  |             |                                                                        |","ref":"readme.html#available-plugins"},{"type":"extras","title":"Grafana Dashboards - README","doc":"Each PromEx plugin comes with a custom tailored Grafana Dashboard. [Click here](https://hexdocs.pm/prom_ex/all.html)\nto check out sample screenshots of each Plugin specific Grafana Dashboard.","ref":"readme.html#grafana-dashboards"},{"type":"extras","title":"Security Concerns - README","doc":"By default, you can set up a Prometheus scrape target without providing any security authorization configuration. As a\nresult, PromEx does not enforce any security precautions by default, and it is up to you to secure your `/metrics`\nendpoint to ensure that people are not seeing sensitive information (sort of like Phoenix LiveDashboard where you need\nto set up your own basic authentication plug to guard access).\n\nThere are a couple of solutions to this problem:\n\n1. If your application is behind a load balancer or an API gateway, you can block access for any external requests to\n   `/metrics` (or whatever route you chose to expose metrics over).\n\n2. If your application is public facing, you can leverage the [Unplug](https://hex.pm/packages/unplug) library that I\n   maintain in order to only execute the `PromEx.Plug` plug when the incoming request fulfills your configured\n   requirements (see the [PromEx.Plug HexDocs](https://hexdocs.pm/prom_ex/1.1.1/PromEx.Plug.html) for an example).\n\n3. If Unplug doesn't suit your needs, you can invoke `PromEx.Plug`\n   [conditionally with your own logic](https://hexdocs.pm/plug/1.16.1/Plug.Builder.html#module-conditional-plugs).","ref":"readme.html#security-concerns"},{"type":"extras","title":"Performance Concerns - README","doc":"You may think to yourself that with all these metrics being collected and scraped, that the performance of your\napplication may be negatively impacted. Luckily PromEx is built upon the solid foundation established by the `Telemetry`,\n`TelemetryMetrics`, and the `TelemetryMetricsPrometheus` projects. These libraries were designed to be as lightweight\nand performant as possible. From some basic stress tests that I have run, I have been unable to observe any meaningful\nperformance reduction (thank you OTP and particularly ETS ;)). Below are the results from a recent stress test using\nApacheBench:","ref":"readme.html#performance-concerns"},{"type":"extras","title":"With PromEx metrics collection - README","doc":"```terminal\n$ ./benchmarks/ab-graph.sh -u http://localhost:4000 -n 1000 -c 50 -k\nServer Software:        Cowboy\nServer Hostname:        localhost\nServer Port:            4000\n\nDocument Path:          /\nDocument Length:        3389 bytes\n\nConcurrency Level:      50\nTime taken for tests:   4.144 seconds\nComplete requests:      1000\nFailed requests:        0\nKeep-Alive requests:    1000\nTotal transferred:      4060000 bytes\nHTML transferred:       3389000 bytes\nRequests per second:    241.32 [#/sec] (mean)\nTime per request:       207.191 [ms] (mean)\nTime per request:       4.144 [ms] (mean, across all concurrent requests)\nTransfer rate:          956.81 [Kbytes/sec] received\n\nConnection Times (ms)\n              min  mean[+/-sd] median   max\nConnect:        0    0   0.2      0       1\nProcessing:    39  202  24.3    203     264\nWaiting:       38  202  24.3    203     264\nTotal:         39  202  24.2    203     264\n\nPercentage of the requests served within a certain time (ms)\n  50%    203\n  66%    210\n  75%    215\n  80%    218\n  90%    227\n  95%    237\n  98%    246\n  99%    255\n 100%    264 (longest request)\n```","ref":"readme.html#with-promex-metrics-collection"},{"type":"extras","title":"Without PromEx metrics collection - README","doc":"```terminal\n$ ./benchmarks/ab-graph.sh -u http://localhost:4000 -n 1000 -c 50 -k\nServer Software:        Cowboy\nServer Hostname:        localhost\nServer Port:            4000\n\nDocument Path:          /\nDocument Length:        3389 bytes\n\nConcurrency Level:      50\nTime taken for tests:   4.156 seconds\nComplete requests:      1000\nFailed requests:        0\nKeep-Alive requests:    1000\nTotal transferred:      4060000 bytes\nHTML transferred:       3389000 bytes\nRequests per second:    240.59 [#/sec] (mean)\nTime per request:       207.822 [ms] (mean)\nTime per request:       4.156 [ms] (mean, across all concurrent requests)\nTransfer rate:          953.90 [Kbytes/sec] received\n\nConnection Times (ms)\n              min  mean[+/-sd] median   max\nConnect:        0    0   0.1      0       1\nProcessing:    38  202  23.1    205     267\nWaiting:       37  202  23.1    205     267\nTotal:         38  202  23.0    205     267\n\nPercentage of the requests served within a certain time (ms)\n  50%    205\n  66%    211\n  75%    215\n  80%    219\n  90%    226\n  95%    232\n  98%    238\n  99%    246\n 100%    267 (longest request)\n```","ref":"readme.html#without-promex-metrics-collection"},{"type":"extras","title":"Plotting the stress test results - README","doc":"In the spirit of visualizing performance characteristics, the percentile data from the ApacheBench stress tests has been\noverlaid and plotted using Gnuplot (thanks to\n[apachebench-graphs](https://github.com/juanluisbaptiste/apachebench-graphs) for making Gnuplot-ing a lot more\nstreamlined :)). As we can see, the distributions track each other more or less 1:1 except for the slowest 5-10% of requests where\nwe see a slight performance hit. In other words, 90% of the time there was no measurable performance overhead in the\ninstrumented application.","ref":"readme.html#plotting-the-stress-test-results"},{"type":"extras","title":"Attribution - README","doc":"It wouldn't be right to not include somewhere in this project a \"thank you\" to the various projects and people that\nhelped make this possible:\n\n- The various projects available in [BEAM Telemetry](https://github.com/beam-telemetry)\n- All of the Prometheus libraries that Ilya Khaprov ([@deadtrickster](https://github.com/deadtrickster)) maintains\n- The logo for the project is an edited version of an SVG image from the [unDraw project](https://undraw.co/)\n- Juan Luis Baptiste for the nifty ApacheBench+Gnuplot shell scripts [apachebench-graphs](https://github.com/juanluisbaptiste/apachebench-graphs)","ref":"readme.html#attribution"},{"type":"extras","title":"Writing PromEx Plugins","doc":"# Writing PromEx Plugins\n\nThis guide will walk you through writing a PromEx plugin. Whether this plugin is for a dependent library or for your\ninternal application metrics, the same patterns apply.","ref":"writing-promex-plugins.html"},{"type":"extras","title":"Getting started - Writing PromEx Plugins","doc":"In order for PromEx to be able to load the appropriate metrics from your plugins, your modules need to leverage the\n`PromEx` behaviour. This behaviour defines 3 optional callbacks. Those callbacks are:\n\n- `event_metrics/1`\n- `polling_metrics/1`\n- `manual_metrics/1`\n\nEach of these callbacks is supposed to return a list of metrics of that type. For example, `polling_metrics/1` needs to\nreturn a list of `PromEx.MetricTypes.Polling` structs (a single struct is also an acceptable return). By doing this, your\nplugin can load your metrics and deal with the nuances of each metric type properly. Each of the `MetricTypes` structs\nall have a field called `:metrics`. This field contains a list of all the `Telemetry.Metrics` definitions that were\nprovided to the struct `build` function.","ref":"writing-promex-plugins.html#getting-started"},{"type":"extras","title":"Adding Event Metrics - Writing PromEx Plugins","doc":"To have your custom plugin expose event based metrics, implement a `event_metrics/1` function and build out a collection\nof `Telemetry.Metrics` structs (`distribution`, `counter`, `last_value`, and `sum`). Be sure to look at plugins like\n`PromEx.Plugins.Phoenix` for more in depth examples.\n\n```elixir\ndefmodule MyApp.PromEx.Plugins.MyPhoenix do\n  use PromEx.Plugin\n\n  @impl true\n  def event_metrics(opts) do\n    http_metrics_tags = gen_http_metrics_tags(opts)\n    phoenix_router = get_phoenix_router(opts)\n    phoenix_stop_event = [:phoenix, :endpoint, :stop]\n\n    Event.build(\n      :phoenix_http_event_metrics,\n      [\n        # Capture request duration information\n        distribution(\n          [:phoenix, :http, :request, :duration, :milliseconds],\n          event_name: phoenix_stop_event,\n          measurement: :duration,\n          description: \"The time it takes for the application to respond to HTTP requests.\",\n          reporter_options: [\n            buckets: exponential!(1, 2, 12)\n          ],\n          tag_values: get_conn_tags(phoenix_router),\n          tags: http_metrics_tags,\n          unit: {:native, :millisecond}\n        )\n\n        # Additional event based metrics ...\n      ]\n    )\n  end\nend\n```","ref":"writing-promex-plugins.html#adding-event-metrics"},{"type":"extras","title":"Adding Polling Metrics - Writing PromEx Plugins","doc":"Polling metrics are similar to event metrics in that they require similar fields (`group_name` and `metrics` to be\nspecific). In addition, the `PromEx.MetricTypes.Polling.build/4` function requires an `measurements_mfa` argument which\nspecifies what function will be executed on the polling interval. This function should run `:telemetry.execute/3`\nsomewhere in its function body. Once that event is executed, the corresponding event in the struct will be triggered and\nyou will capture the desired data point. The following example from `PromEx.Plugins.Beam` should highlight this concept:\n\n```elixir\ndefmodule PromEx.Plugins.Beam do\n  use PromEx.Plugin\n\n  @memory_event [:prom_ex, :plugin, :beam, :memory]\n\n  @impl true\n  def polling_metrics(opts) do\n    poll_rate = Keyword.get(opts, :poll_rate, 5_000)\n\n    [\n      memory_metrics(poll_rate)\n    ]\n  end\n\n  defp memory_metrics(poll_rate) do\n    Polling.build(\n      :beam_memory_polling_events,\n      poll_rate,\n      {__MODULE__, :execute_memory_metrics, []},\n      [\n        # Capture the total memory allocated to the entire Erlang VM (or BEAM for short)\n        last_value(\n          [:beam, :memory, :total, :kilobytes],\n          event_name: @memory_event,\n          description: \"The total amount of memory currently allocated.\",\n          measurement: :total,\n          unit: {:byte, :kilobyte}\n        )\n\n        # More memory metrics here\n      ]\n    )\n  end\n\n  @doc false\n  def execute_memory_metrics do\n    memory_measurements =\n      :erlang.memory()\n      |> Map.new()\n\n    :telemetry.execute(@memory_event, memory_measurements, %{})\n  end\nend\n```\n\nDepending on what `:poll_rate` value you pass to the initialization tuple for `PromEx.Plugins.Beam`, the\n`execute_memory_metrics/0` function will be execute on that specified interval.","ref":"writing-promex-plugins.html#adding-polling-metrics"},{"type":"extras","title":"Adding Manual Metrics - Writing PromEx Plugins","doc":"Manual metrics behave more or less the same as polling metrics except they do not require a poll rate value. Instead the\nprovided `measurements_mfa` is called once on application start, and the metrics are only then updated if you make a\ncall to `PromEx.ManualMetricsManager.refresh_metrics/1`. An example of this can be seen from the\n`PromEx.Plugins.Application` plugin:\n\n```elixir\ndefmodule PromEx.Plugins.Application do\n  use PromEx.Plugin\n\n  @impl true\n  def manual_metrics(opts) do\n    otp_app = Keyword.fetch!(opts, :otp_app)\n    apps = Keyword.get(opts, :deps, :all)\n\n    Manual.build(\n      :application_versions_manual_metrics,\n      {__MODULE__, :apps_running, [otp_app, apps]},\n      [\n        # Capture information regarding the primary application (i.e the user's application)\n        last_value(\n          [otp_app | [:application, :primary, :info]],\n          event_name: [otp_app | [:application, :primary, :info]],\n          description: \"Information regarding the primary application.\",\n          measurement: :status,\n          tags: [:name, :version, :modules]\n        )\n\n        # Additional metrics here\n      ]\n    )\n  end\n\n  @doc false\n  def apps_running(otp_app, apps) do\n    ...\n\n    # Emit primary app details\n    :telemetry.execute(\n      [otp_app | [:application, :primary, :info]],\n      %{\n        status: if(Map.has_key?(started_apps, otp_app), do: 1, else: 0)\n      },\n      %{\n        name: otp_app,\n        version:\n          Map.get_lazy(started_apps, otp_app, fn ->\n            Map.get(loaded_only_apps, otp_app, \"undefined\")\n          end),\n        modules: length(Application.spec(otp_app)[:modules])\n      }\n    )\n  end\nend\n```\n\nSo in this example, `apps_running/2` is the function that is denoted by the MFA and will be called once automatically on\napplication start, but then at that point it is up to the user to refresh the data point.","ref":"writing-promex-plugins.html#adding-manual-metrics"},{"type":"extras","title":"Introduction to Telemetry","doc":"# Introduction to Telemetry\n\nThis section serves as a quick primer for the Telemetry library, how it works, and how you can leverage it in your\napplications. While there is nothing specific to PromEx here, but it is important to know how Telemetry works so that\nyou can effectively create your own PromEx plugins.","ref":"telemetry.html"},{"type":"extras","title":"What is Telemetry? - Introduction to Telemetry","doc":"At a high level, Telemetry offers a means for libraries and applications to surface internal events. These events can\nbe emitted prior to starting some logical body of work, after the work has completed, after the processing of the work\nhas resulted in an exception, or even any time you want to denote that something of importance has occurred. As an\nexample, these events can include the time it takes to process incoming HTTP requests (like in Phoenix) or the time\nit takes to process a job (like in Broadway & Oban) and well as metadata related to the event (like the requested route\nin Phoenix).","ref":"telemetry.html#what-is-telemetry"},{"type":"extras","title":"How Does it Work? - Introduction to Telemetry","doc":"Under the hood, Telemetry works by attaching callback functions to events (tracked in ETS), and then invoking those\nfunctions serially whenever that event occurs. In other words, any time an event is triggered, Telemetry dynamically\ndispatches to each of the callbacks that are registered for that particular event. You can think of this as effectively\nbeing a pub/sub style library but synchronous as opposed to asynchronous. Given that the callbacks are executed in a\nsynchronous fashion, it is highly recommended that you keep your callbacks very lightweight and functionally limited. If\nadditional blocking work needs to take place as a result of an event, be sure to spin that work off into its own Task or\npass it along to a separate GenServer.","ref":"telemetry.html#how-does-it-work"},{"type":"extras","title":"Why is Telemetry Important? - Introduction to Telemetry","doc":"In my opinion, Telemetry (and the other repositories in the beam-telemetry GitHub organization) provides two key benefits\nto the Erlang, Elixir and Beam communities.\n\nFirstly, Telemetry provides a consistent interface through which applications and libraries can expose internal events.\nThis consistent interface consists of measurements and metadata that are attached to each event that can then be used by\nconsumers in whichever way fits the user's needs. For example, using the same Telemetry event, you can produce a custom\nlog message and even metrics by attaching two separate callback functions to the desired Telemetry event.\n\nSecondly, attaching callbacks to Telemetry events is a very low friction operation and can be done without much\nceremony. This is important given that if the ergonomics of Telemetry were cumbersome to work with, library authors\nwould not be inclined to leverage Telemetry for surfacing internal events. Specifically, Telemetry does not require\nany global setup or configuration by either the library author or the user. Instead, Telemetry will sort out the dynamic\nfunction dispatch through ETS tables that it manages and will only invoke callback functions when callbacks have been\nattached to a particular event.\n\nAs a result of these two points (and others), it is no surprise that over 100 libraries in the Elixir and Erlang\necosystem have adopted Telemetry as their primary means of surfacing internal events.","ref":"telemetry.html#why-is-telemetry-important"},{"type":"extras","title":"How Can I Use Telemetry in my Project? - Introduction to Telemetry","doc":"As previously mentioned, leveraging Telemetry consists of a library (or your application) executing an event, and you as\nthe user attaching a callback to said event. As an example, let's suppose that we want to emit an event any time a user\nregisters for our service. In our registration function we could do something like so in order to capture when users are\nsuccessfully created and when errors are encountered:\n\n```elixir\ndef register_user(attrs) do\n  %User{}\n  |> User.registration_changeset(updated_attrs)\n  |> Repo.insert()\n  |> case do\n    {:ok, new_user} = result ->\n      :telemetry.execute([:my_cool_app, :accounts, :new_user, :success], %{}, %{user: new_user})\n      result\n\n    {:error, changeset} = error ->\n      :telemetry.execute([:my_cool_app, :accounts, :new_user, :error], %{}, %{error: changeset})\n      error\n  end\nend\n```\n\nElsewhere in your application code (perhaps where you initialize your application) you can attach to these events by\ndoing the following:\n\n```elixir\n:telemetry.attach(\n  \"my-handler-1\",\n  [:my_cool_app, :accounts, :new_user, :success],\n  fn _event_name, _event_measurement, event_metadata, _config ->\n    Logger.debug(\"User has registered: #{inspect(event_metadata)}\")\n  end,\n  %{}\n)\n\n:telemetry.attach(\n  \"my-handler-2\",\n  [:my_cool_app, :accounts, :new_user, :error],\n  fn _event_name, _event_measurement, event_metadata, _config ->\n    Logger.warning(\"User failed to register: #{inspect(event_metadata)}\")\n  end,\n  %{}\n)\n```\n\nNow, any time an error occurs and a user fails to register a warning log is generated with the metadata related to the\nerror. Similarly, a debug log message is created any time a user is successfully created.","ref":"telemetry.html#how-can-i-use-telemetry-in-my-project"},{"type":"extras","title":"Additional Resources - Introduction to Telemetry","doc":"- https://github.com/beam-telemetry/telemetry\n- https://hexdocs.pm/telemetry_metrics/Telemetry.Metrics.html\n- https://keathley.io/blog/telemetry-conventions.html","ref":"telemetry.html#additional-resources"},{"type":"extras","title":"Running Multiple Agents","doc":"# Running Multiple Agents\n\nSuppose you want to send your metrics to two separate Prometheus databases, and define two separate sets of dashboards - one for the ops team, and one for the business team.\nYou can do this by creating two PromEx configurations.\n\nTo generate distinct PromEx modules:\n\n```sh\nmix prom_ex.gen.config -d ops -m PromExOps\nmix prom_ex.gen.config -d biz -m PromExBiz\n```\n\nEach PromEx config will run its own Grafana Agent. You need to configure `working_directory`, `agent_port` and `grpc_port` to make sure they don't collide:\n\n```elixir\nconfig :my_app, MyApp.PromExOps,\n  grafana_agent: [\n    working_directory: System.fetch_env!(\"RELEASE_TMP\") <> \"/grafana-ops\",\n    config_opts: [\n      ...\n      agent_port: 4040,\n      grpc_port: 9040\n    ]\n  ]\n\nconfig :my_app, MyApp.PromExBiz,\n  grafana_agent: [\n    working_directory: System.fetch_env!(\"RELEASE_TMP\") <> \"/grafana-biz\",\n    config_opts: [\n      ...\n      agent_port: 4041,\n      grpc_port: 9041\n    ]\n  ]\n```\n\nAdd each module to your application's supervisor per the directions in the generated PromEx file.","ref":"running-multiple-agents.html"},{"type":"extras","title":"Endpoints - Running Multiple Agents","doc":"You can configure each agent to scrape the same set of metrics:\n\n```elixir\n# endpoint.ex\nplug PromEx.Plug, prom_ex_module: MyApp.PromExOps\n\n# in mix config, set `metrics_server_path: \"/metrics\"`\n```\n\nor define separate endpoints:\n\n```elixir\nplug PromEx.Plug, prom_ex_module: MyApp.PromExOps, path: \"/metrics/ops\"\nplug PromEx.Plug, prom_ex_module: MyApp.PromExBiz, path: \"/metrics/biz\"\n```","ref":"running-multiple-agents.html#endpoints"},{"type":"extras","title":"Seeding Metrics","doc":"# Seeding Metrics\n\nIn some applications, telemetry is only emitted periodically or in spurts. Capturing\nthese metrics via `c:PromEx.Plugin.event_metrics/1` can lead to gaps in Grafana graphs\nafter a service restart or deployment.\n\nWhen an application starts, its Prometheus metrics are empty; its `metrics_server_path`\nwill only show metrics that have been emitted and captured by `PromEx` over the course\nof the beam's lifespan. When a metric is not seen within a time period, Prometheus\nreturns no data for that metric, and in Grafana rather than data points with `0` values,\nthe graph will show `No Data`.\n\nOne mechanism to ensure that the graphs are populated is to seed the event metrics, i.e.\nemit them at least once after `PromEx` has started. This can be done via a\n`c:Application.start_phase/3`.","ref":"seeding-metrics.html"},{"type":"extras","title":"Start Phases - Seeding Metrics","doc":"Start phases are configured in a project's `application` callback in `mix.exs`:\n\n```elixir\n  def application do\n    [\n      extra_applications: [],\n      mod: {MyApp.Application, []},\n      start_phases: [seed_prom_ex_telemetry: []]\n    ]\n  end\n```\n\nThis requires adding a callback to `MyApp.Application`:\n\n```elixir\ndefmodule MyApp.Application do\n  use Application\n\n  @impl true\n  def start(_type, _args) do\n    children = [\n      MyApp.PromEx\n      # ... Repo, Phoenix, etc\n    ]\n\n    opts = [strategy: :one_for_one, name: MyApp.Supervisor]\n    Supervisor.start_link(children, opts)\n  end\n\n  @impl true\n  def start_phase(:seed_prom_ex_telemetry, :normal, _) do\n    MyApp.PromEx.seed_event_metrics()\n    :ok\n  end\nend\n```\n\nThe `seed_event_metrics/0` function can send any telemetry events desired, with any\nvariations in measurements and metadata to ensure that the `metrics_server_path` is\nfully populated when it is first scraped.\n\n```elixir\ndef seed_event_metrics do\n  # :telemetry.execute([:my_app, :event], %{}, %{status: :ok})\n  # :telemetry.execute([:my_app, :event], %{}, %{status: :error})\n  # :telemetry.execute([:my_app, :event_span, :start], %{count: 1}, %{})\n  # :telemetry.execute([:my_app, :event_span, :stop], %{count: 1}, %{})\n  # :telemetry.execute([:my_app, :event_span, :exception], %{count: 1}, %{})\n\n  :ok\nend\n```\n\nOne thing to note is that for counter metrics, seeding the events in this way will\nresult in the values being initialized to a count of `1`. In Grafana graphs showing\ncounters over time, the value resetting to *either* `0` or `1` will result in a negative\ndelta; one may want to combine `idelta` with `clamp_min`\n(i.e. `clamp_min(idelta( [$__rate_interval]), 0)`) to ensure that the\ninitial value after a restart appears in the graph as no change.","ref":"seeding-metrics.html#start-phases"},{"type":"extras","title":"Dashboards Screenshots","doc":"# Dashboards Screenshots","ref":"all.html"},{"type":"extras","title":"Application - Dashboards Screenshots","doc":"The application dashboard surfaces static information regarding the application. Things such as the dependencies of the\napplication, GIT SHA+author and uptime.","ref":"all.html#application"},{"type":"extras","title":"BEAM - Dashboards Screenshots","doc":"The BEAM dashboard presents information regarding the Erlang virtual machine. Things such as memory statistics,\nprocess and atom counts, scheduler information, and much more","ref":"all.html#beam"},{"type":"extras","title":"Ecto - Dashboards Screenshots","doc":"The Ecto dashboard contains charts to track query execution time, number of results returned, and static information\nregarding the configured Repo.","ref":"all.html#ecto"},{"type":"extras","title":"Oban - Dashboards Screenshots","doc":"The Oban plugin presents information regarding the job execution time, job queue time, available jobs in queue, static\nconfiguration settings and much, much more.","ref":"all.html#oban"},{"type":"extras","title":"Phoenix - Dashboards Screenshots","doc":"The Phoenix dashboard presents information regarding HTTP requests and channel connections.","ref":"all.html#phoenix"},{"type":"extras","title":"Phoenix LiveView - Dashboards Screenshots","doc":"The Phoenix LiveView dashboard presents information regarding the various LiveView callbacks and the time it takes to\nexecute them.","ref":"all.html#phoenix-liveview"},{"type":"extras","title":"Broadway - Dashboards Screenshots","doc":"The Broadway dashboard presents information regarding the messages the Broadway has processed as well as how\nit has processed message batches.","ref":"all.html#broadway"}],"producer":{"name":"ex_doc","version":[48,46,51,52,46,50]},"content_type":"text/markdown"}